import sys
from dataclasses import dataclass
from datetime import date, datetime, timezone
from enum import Enum, auto
from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    Generic,
    List,
    Literal,
    Optional,
    Tuple,
    TypedDict,
    TypeVar,
    Union,  # pyright: ignore
)

import annotated_types
import msgspec
import pytest
from msgspec import Struct
from typing_extensions import Annotated, TypeAlias, TypeAliasType

from litestar import Controller, MediaType, get, post
from litestar._openapi.schema_generation.plugins import openapi_schema_plugins
from litestar._openapi.schema_generation.schema import (
    KWARG_DEFINITION_ATTRIBUTE_TO_OPENAPI_PROPERTY_MAP,
    SchemaCreator,
)
from litestar.app import DEFAULT_OPENAPI_CONFIG, Litestar
from litestar.di import Provide
from litestar.enums import ParamType
from litestar.exceptions import ImproperlyConfiguredException
from litestar.openapi.spec import ExternalDocumentation, OpenAPIType, Reference
from litestar.openapi.spec.example import Example
from litestar.openapi.spec.parameter import Parameter as OpenAPIParameter
from litestar.openapi.spec.schema import Schema
from litestar.pagination import ClassicPagination, CursorPagination, OffsetPagination
from litestar.params import KwargDefinition, Parameter, ParameterKwarg
from litestar.testing import create_test_client
from litestar.typing import FieldDefinition
from litestar.utils.helpers import get_name
from tests.helpers import get_schema_for_field_definition
from tests.models import DataclassPerson, DataclassPet

if TYPE_CHECKING:
    from types import ModuleType
    from typing import Callable

T = TypeVar("T")


def test_process_schema_result() -> None:
    test_str = "abc"
    kwarg_definition = ParameterKwarg(
        examples=[Example(value=1)],
        external_docs=ExternalDocumentation(url="https://example.com/docs"),
        content_encoding="utf-8",
        default=test_str,
        title=test_str,
        description=test_str,
        const=True,
        gt=1,
        ge=1,
        lt=1,
        le=1,
        multiple_of=1,
        min_items=1,
        max_items=1,
        min_length=1,
        max_length=1,
        pattern="^[a-z]$",
    )
    schema = get_schema_for_field_definition(
        FieldDefinition.from_annotation(annotation=str, kwarg_definition=kwarg_definition)
    )

    assert schema.title
    assert schema.const == test_str
    assert kwarg_definition.examples
    for signature_key, schema_key in KWARG_DEFINITION_ATTRIBUTE_TO_OPENAPI_PROPERTY_MAP.items():
        if schema_key == "examples":
            assert schema.examples == [kwarg_definition.examples[0].value]
        else:
            assert getattr(schema, schema_key) == getattr(kwarg_definition, signature_key)


def test_override_schema_component_key() -> None:
    @dataclass
    class Data:
        pass

    @post("/")
    def handler(
        data: Data,
    ) -> Annotated[Data, Parameter(schema_component_key="not_data")]:
        return Data()

    @get("/")
    def handler_2() -> Annotated[Data, Parameter(schema_component_key="not_data")]:
        return Data()

    app = Litestar([handler, handler_2])
    schema = app.openapi_schema.to_schema()
    # we expect the annotated / non-annotated to generate independent components
    assert schema["paths"]["/"]["post"]["requestBody"]["content"]["application/json"] == {
        "schema": {"$ref": "#/components/schemas/test_override_schema_component_key.Data"}
    }
    assert schema["paths"]["/"]["post"]["responses"]["201"]["content"] == {
        "application/json": {"schema": {"$ref": "#/components/schemas/not_data"}}
    }
    # a response with the same type and the same name should reference the same component
    assert schema["paths"]["/"]["get"]["responses"]["200"]["content"] == {
        "application/json": {"schema": {"$ref": "#/components/schemas/not_data"}}
    }
    assert app.openapi_schema.to_schema()["components"] == {
        "schemas": {
            "not_data": {"properties": {}, "type": "object", "required": [], "title": "Data"},
            "test_override_schema_component_key.Data": {
                "properties": {},
                "type": "object",
                "required": [],
                "title": "Data",
            },
        }
    }


def test_override_schema_component_key_raise_if_keys_are_not_unique() -> None:
    @dataclass
    class Data:
        pass

    @dataclass
    class Data2:
        pass

    @post("/")
    def handler(
        data: Data,
    ) -> Annotated[Data, Parameter(schema_component_key="not_data")]:
        return Data()

    @get("/")
    def handler_2() -> Annotated[Data2, Parameter(schema_component_key="not_data")]:
        return Data2()

    with pytest.raises(ImproperlyConfiguredException, match="Schema component keys must be unique"):
        Litestar([handler, handler_2]).openapi_schema.to_schema()


def test_dependency_schema_generation() -> None:
    async def top_dependency(query_param: int) -> int:
        return query_param

    async def mid_level_dependency(header_param: str = Parameter(header="header_param", required=False)) -> int:
        return 5

    async def local_dependency(path_param: int, mid_level: int, top_level: int) -> int:
        return path_param + mid_level + top_level

    class MyController(Controller):
        path = "/test"
        dependencies = {"mid_level": Provide(mid_level_dependency)}

        @get(
            path="/{path_param:int}",
            dependencies={
                "summed": Provide(local_dependency),
            },
            media_type=MediaType.TEXT,
        )
        def test_function(self, summed: int, handler_param: int) -> str:
            return str(summed)

    with create_test_client(
        MyController,
        dependencies={"top_level": Provide(top_dependency)},
        openapi_config=DEFAULT_OPENAPI_CONFIG,
    ) as client:
        handler = client.app.openapi_schema.paths["/test/{path_param}"]
        data = {param.name: {"in": param.param_in, "required": param.required} for param in handler.get.parameters}
        assert data == {
            "path_param": {"in": ParamType.PATH, "required": True},
            "header_param": {"in": ParamType.HEADER, "required": False},
            "query_param": {"in": ParamType.QUERY, "required": True},
            "handler_param": {"in": ParamType.QUERY, "required": True},
        }


def test_get_schema_for_annotation_enum() -> None:
    class Opts(str, Enum):
        opt1 = "opt1"
        opt2 = "opt2"

    schema = get_schema_for_field_definition(FieldDefinition.from_annotation(Opts))
    assert schema.enum == ["opt1", "opt2"]


ValueType: TypeAlias = Literal["a", "b", "c"]
ConstType: TypeAlias = Literal[1]


def test_handling_of_literals() -> None:
    @dataclass
    class DataclassWithLiteral:
        value: ValueType
        const: ConstType
        composite: Literal[ValueType, ConstType]

    schema = get_schema_for_field_definition(FieldDefinition.from_kwarg(name="", annotation=DataclassWithLiteral))

    assert isinstance(schema, Schema)
    assert schema.properties

    value = schema.properties["value"]
    assert isinstance(value, Schema)
    assert value.enum == ["a", "b", "c"]

    const = schema.properties["const"]
    assert isinstance(const, Schema)
    assert const.const == 1

    composite = schema.properties["composite"]
    assert isinstance(composite, Schema)
    assert composite.enum == ["a", "b", "c", 1]


def test_schema_hashing() -> None:
    schema = Schema(
        one_of=[
            Schema(type=OpenAPIType.STRING),
            Schema(type=OpenAPIType.NUMBER),
            Schema(type=OpenAPIType.OBJECT, properties={"key": Schema(type=OpenAPIType.STRING)}),
        ],
        examples=[None, [1, 2, 3]],
    )
    assert hash(schema)


def test_title_validation() -> None:
    # TODO: what is this actually testing?
    creator = SchemaCreator(plugins=openapi_schema_plugins)
    person_ref = creator.for_field_definition(FieldDefinition.from_kwarg(name="Person", annotation=DataclassPerson))
    pet_ref = creator.for_field_definition(FieldDefinition.from_kwarg(name="Pet", annotation=DataclassPet))
    assert isinstance(person_ref, Reference)
    assert isinstance(pet_ref, Reference)
    assert isinstance(creator.schema_registry.from_reference(person_ref).schema, Schema)
    assert isinstance(creator.schema_registry.from_reference(pet_ref).schema, Schema)


@pytest.mark.parametrize("with_future_annotations", [True, False])
def test_create_schema_for_dataclass_with_annotated_model_attribute(
    with_future_annotations: bool, create_module: "Callable[[str], ModuleType]"
) -> None:
    """Test that a model with an annotated attribute is correctly handled."""
    module = create_module(
        f"""
{"from __future__ import annotations" if with_future_annotations else ""}
from typing_extensions import Annotated
from dataclasses import dataclass

@dataclass
class Foo:
    foo: Annotated[int, "Foo description"]
"""
    )
    schema = get_schema_for_field_definition(FieldDefinition.from_annotation(module.Foo))
    assert schema.properties and "foo" in schema.properties


@pytest.mark.parametrize("with_future_annotations", [True, False])
def test_create_schema_for_typedict_with_annotated_required_and_not_required_model_attributes(
    with_future_annotations: bool, create_module: "Callable[[str], ModuleType]"
) -> None:
    """Test that a model with an annotated attribute is correctly handled."""
    module = create_module(
        f"""
{"from __future__ import annotations" if with_future_annotations else ""}
from typing_extensions import Annotated, Required, NotRequired
from typing import TypedDict

class Foo(TypedDict):
    foo: Annotated[int, "Foo description"]
    bar: Annotated[Required[int], "Bar description"]
    baz: Annotated[NotRequired[int], "Baz description"]
"""
    )
    schema = get_schema_for_field_definition(FieldDefinition.from_annotation(module.Foo))
    assert schema.properties and all(key in schema.properties for key in ("foo", "bar", "baz"))


def test_create_schema_from_msgspec_annotated_type() -> None:
    class Lookup(msgspec.Struct):
        int_field: Annotated[int, msgspec.Meta(gt=0)]
        str_field: Annotated[
            str,
            msgspec.Meta(max_length=16, examples=["example"], description="description", title="title", pattern=r"\w+"),
        ]
        bytes_field: Annotated[bytes, msgspec.Meta(max_length=2, min_length=1)]
        default_field: Annotated[str, msgspec.Meta(min_length=1)] = "a"

    schema = get_schema_for_field_definition(FieldDefinition.from_kwarg(name="Lookup", annotation=Lookup))

    assert schema.properties["int_field"].type == OpenAPIType.INTEGER  # type: ignore[index, union-attr]
    assert schema.properties["int_field"].exclusive_minimum == 0  # type: ignore[index, union-attr]
    assert schema.properties["str_field"].type == OpenAPIType.STRING  # type: ignore[index, union-attr]
    assert schema.properties["str_field"].examples == ["example"]  # type: ignore[index, union-attr]
    assert schema.properties["str_field"].description == "description"  # type: ignore[index]
    assert schema.properties["str_field"].title == "title"  # type: ignore[index, union-attr]
    assert schema.properties["str_field"].max_length == 16  # type: ignore[index, union-attr]
    assert sorted(schema.required) == sorted(["int_field", "str_field", "bytes_field"])  # type: ignore[arg-type]
    assert schema.properties["bytes_field"].to_schema() == {  # type: ignore[index]
        "contentEncoding": "utf-8",
        "maxLength": 2,
        "minLength": 1,
        "type": "string",
    }


def test_annotated_types() -> None:
    historical_date = date(year=1980, day=1, month=1)
    today = date.today()

    @dataclass
    class MyDataclass:
        constrained_int: Annotated[int, annotated_types.Gt(1), annotated_types.Lt(10)]
        constrained_float: Annotated[float, annotated_types.Ge(1), annotated_types.Le(10)]
        constrained_date: Annotated[date, annotated_types.Interval(gt=historical_date, lt=today)]
        constrained_lower_case: annotated_types.LowerCase[str]
        constrained_upper_case: annotated_types.UpperCase[str]
        constrained_is_ascii: annotated_types.IsAscii[str]
        constrained_is_digit: annotated_types.IsDigit[str]

    schema = get_schema_for_field_definition(FieldDefinition.from_kwarg(name="MyDataclass", annotation=MyDataclass))

    assert schema.properties["constrained_int"].exclusive_minimum == 1  # type: ignore[index, union-attr]
    assert schema.properties["constrained_int"].exclusive_maximum == 10  # type: ignore[index, union-attr]
    assert schema.properties["constrained_float"].minimum == 1  # type: ignore[index, union-attr]
    assert schema.properties["constrained_float"].maximum == 10  # type: ignore[index, union-attr]
    assert datetime.fromtimestamp(
        schema.properties["constrained_date"].exclusive_minimum,  # type: ignore[arg-type, index, union-attr]
        tz=timezone.utc,
    ) == datetime.fromordinal(historical_date.toordinal()).replace(tzinfo=timezone.utc)
    assert datetime.fromtimestamp(
        schema.properties["constrained_date"].exclusive_maximum,  # type: ignore[arg-type, index, union-attr]
        tz=timezone.utc,
    ) == datetime.fromordinal(today.toordinal()).replace(tzinfo=timezone.utc)
    assert schema.properties["constrained_lower_case"].description == "must be in lower case"  # type: ignore[index]
    assert schema.properties["constrained_upper_case"].description == "must be in upper case"  # type: ignore[index]
    assert schema.properties["constrained_is_ascii"].pattern == "[[:ascii:]]"  # type: ignore[index, union-attr]
    assert schema.properties["constrained_is_digit"].pattern == "[[:digit:]]"  # type: ignore[index, union-attr]


def test_literal_enums() -> None:
    class Foo(Enum):
        A = auto()
        B = auto()

    schema = get_schema_for_field_definition(FieldDefinition.from_annotation(List[Literal[Foo.A]]))
    assert isinstance(schema.items, Schema)
    assert schema.items.const == 1


@dataclass
class DataclassGeneric(Generic[T]):
    foo: T
    optional_foo: Optional[T]
    annotated_foo: Annotated[T, object()]


class MsgspecGeneric(Struct, Generic[T]):
    foo: T
    optional_foo: Optional[T]
    annotated_foo: Annotated[T, object()]


annotations: List[type] = [DataclassGeneric[int], MsgspecGeneric[int]]

# Generic TypedDict was only supported from 3.11 onwards
if sys.version_info >= (3, 11):

    class TypedDictGeneric(TypedDict, Generic[T]):
        foo: T
        optional_foo: Optional[T]
        annotated_foo: Annotated[T, object()]

    annotations.append(TypedDictGeneric[int])


@pytest.mark.parametrize("cls", annotations)
def test_schema_generation_with_generic_classes(cls: Any) -> None:
    expected_foo_schema = Schema(type=OpenAPIType.INTEGER)
    expected_optional_foo_schema = Schema(one_of=[Schema(type=OpenAPIType.INTEGER), Schema(type=OpenAPIType.NULL)])

    properties = get_schema_for_field_definition(
        FieldDefinition.from_kwarg(name=get_name(cls), annotation=cls)
    ).properties
    assert properties
    assert properties["foo"] == expected_foo_schema
    assert properties["annotated_foo"] == expected_foo_schema
    assert properties["optional_foo"] == expected_optional_foo_schema


B = TypeVar("B", bound=int)
C = TypeVar("C", int, str)


@dataclass
class ConstrainedGenericDataclass(Generic[T, B, C]):
    bound: B
    constrained: C
    union: Union[T, bool]
    union_constrained: Union[C, bool]
    union_bound: Union[B, bool]


def test_schema_generation_with_generic_classes_constrained() -> None:
    cls = ConstrainedGenericDataclass
    properties = get_schema_for_field_definition(
        FieldDefinition.from_kwarg(name=cls.__name__, annotation=cls)
    ).properties

    assert properties
    assert properties["bound"] == Schema(type=OpenAPIType.INTEGER)
    assert properties["constrained"] == Schema(
        one_of=[Schema(type=OpenAPIType.INTEGER), Schema(type=OpenAPIType.STRING)]
    )
    assert properties["union"] == Schema(one_of=[Schema(type=OpenAPIType.OBJECT), Schema(type=OpenAPIType.BOOLEAN)])
    assert properties["union_constrained"] == Schema(
        one_of=[Schema(type=OpenAPIType.INTEGER), Schema(type=OpenAPIType.STRING), Schema(type=OpenAPIType.BOOLEAN)]
    )
    assert properties["union_bound"] == Schema(
        one_of=[Schema(type=OpenAPIType.INTEGER), Schema(type=OpenAPIType.BOOLEAN)]
    )


@pytest.mark.parametrize(
    "annotation",
    (
        ClassicPagination[DataclassGeneric[int]],
        OffsetPagination[DataclassGeneric[int]],
        CursorPagination[int, DataclassGeneric[int]],
    ),
)
def test_schema_generation_with_pagination(annotation: Any) -> None:
    expected_foo_schema = Schema(type=OpenAPIType.INTEGER)
    expected_optional_foo_schema = Schema(one_of=[Schema(type=OpenAPIType.INTEGER), Schema(type=OpenAPIType.NULL)])

    properties = get_schema_for_field_definition(FieldDefinition.from_annotation(annotation).inner_types[-1]).properties

    assert properties
    assert properties["foo"] == expected_foo_schema
    assert properties["annotated_foo"] == expected_foo_schema
    assert properties["optional_foo"] == expected_optional_foo_schema


def test_schema_generation_with_ellipsis() -> None:
    schema = get_schema_for_field_definition(FieldDefinition.from_annotation(Tuple[int, ...]))
    assert isinstance(schema.items, Schema)
    assert schema.items.type == OpenAPIType.INTEGER


def test_schema_tuple_with_union() -> None:
    schema = get_schema_for_field_definition(FieldDefinition.from_annotation(Tuple[int, Union[int, str]]))
    assert schema.prefix_items == [
        Schema(type=OpenAPIType.INTEGER),
        Schema(one_of=[Schema(type=OpenAPIType.INTEGER), Schema(type=OpenAPIType.STRING)]),
    ]


def test_schema_tuple() -> None:
    schema = get_schema_for_field_definition(FieldDefinition.from_annotation(Tuple[int, str]))
    assert schema == Schema(
        prefix_items=[Schema(type=OpenAPIType.INTEGER), Schema(type=OpenAPIType.STRING)],
        type=OpenAPIType.ARRAY,
    )


def test_schema_optional_tuple() -> None:
    schema = get_schema_for_field_definition(FieldDefinition.from_annotation(Optional[Tuple[int, str]]))
    assert schema == Schema(
        one_of=[
            Schema(
                prefix_items=[Schema(type=OpenAPIType.INTEGER), Schema(type=OpenAPIType.STRING)],
                type=OpenAPIType.ARRAY,
            ),
            Schema(type=OpenAPIType.NULL),
        ],
    )


def test_optional_enum() -> None:
    class Foo(Enum):
        A = 1
        B = "b"

    creator = SchemaCreator(plugins=openapi_schema_plugins)
    schema = creator.for_field_definition(FieldDefinition.from_annotation(Optional[Foo]))
    assert isinstance(schema, Schema)
    assert schema.type is None
    assert schema.one_of is not None
    null_schema = schema.one_of[1]
    assert isinstance(null_schema, Schema)
    assert null_schema.type is not None
    assert null_schema.type is OpenAPIType.NULL
    enum_ref = schema.one_of[0]
    assert isinstance(enum_ref, Reference)
    assert enum_ref.ref == "#/components/schemas/tests_unit_test_openapi_test_schema_test_optional_enum.Foo"
    enum_schema = creator.schema_registry.from_reference(enum_ref).schema
    assert enum_schema.type
    assert set(enum_schema.type) == {OpenAPIType.INTEGER, OpenAPIType.STRING}
    assert enum_schema.enum
    assert enum_schema.enum[0] == 1
    assert enum_schema.enum[1] == "b"


def test_optional_str_specified_enum() -> None:
    class StringEnum(str, Enum):
        A = "a"
        B = "b"

    creator = SchemaCreator(plugins=openapi_schema_plugins)
    schema = creator.for_field_definition(FieldDefinition.from_annotation(Optional[StringEnum]))
    assert isinstance(schema, Schema)
    assert schema.type is None
    assert schema.one_of is not None

    enum_ref = schema.one_of[0]
    assert isinstance(enum_ref, Reference)
    assert (
        enum_ref.ref
        == "#/components/schemas/tests_unit_test_openapi_test_schema_test_optional_str_specified_enum.StringEnum"
    )
    enum_schema = creator.schema_registry.from_reference(enum_ref).schema
    assert enum_schema.type
    assert enum_schema.type == OpenAPIType.STRING
    assert enum_schema.enum
    assert enum_schema.enum[0] == "a"
    assert enum_schema.enum[1] == "b"

    null_schema = schema.one_of[1]
    assert isinstance(null_schema, Schema)
    assert null_schema.type is not None
    assert null_schema.type is OpenAPIType.NULL


def test_optional_int_specified_enum() -> None:
    class IntEnum(int, Enum):
        A = 1
        B = 2

    creator = SchemaCreator(plugins=openapi_schema_plugins)
    schema = creator.for_field_definition(FieldDefinition.from_annotation(Optional[IntEnum]))
    assert isinstance(schema, Schema)
    assert schema.type is None
    assert schema.one_of is not None

    enum_ref = schema.one_of[0]
    assert isinstance(enum_ref, Reference)
    assert (
        enum_ref.ref
        == "#/components/schemas/tests_unit_test_openapi_test_schema_test_optional_int_specified_enum.IntEnum"
    )
    enum_schema = creator.schema_registry.from_reference(enum_ref).schema
    assert enum_schema.type
    assert enum_schema.type == OpenAPIType.INTEGER
    assert enum_schema.enum
    assert enum_schema.enum[0] == 1
    assert enum_schema.enum[1] == 2

    null_schema = schema.one_of[1]
    assert isinstance(null_schema, Schema)
    assert null_schema.type is not None
    assert null_schema.type is OpenAPIType.NULL


def test_optional_literal() -> None:
    schema = get_schema_for_field_definition(FieldDefinition.from_annotation(Optional[Literal[1]]))
    assert schema.type is not None
    assert set(schema.type) == {OpenAPIType.INTEGER, OpenAPIType.NULL}
    assert schema.enum == [1, None]


def test_not_generating_examples_property() -> None:
    with_examples = SchemaCreator(generate_examples=True)
    without_examples = with_examples.not_generating_examples
    assert without_examples.generate_examples is False


def test_process_schema_result_with_unregistered_object_schema() -> None:
    """This test ensures that if a schema is created for an object and not registered in the schema registry, the
    schema is returned as-is, and not referenced.
    """
    schema = Schema(title="has title", type=OpenAPIType.OBJECT)
    field_definition = FieldDefinition.from_annotation(dict)
    assert SchemaCreator().process_schema_result(field_definition, schema) is schema


@pytest.mark.parametrize("base_type", [msgspec.Struct, TypedDict, dataclass])
def test_type_union(base_type: type) -> None:
    if base_type is dataclass:

        @dataclass
        class ModelA:  # pyright: ignore
            pass

        @dataclass
        class ModelB:  # pyright: ignore
            pass

    else:

        class ModelA(base_type):  # type: ignore[no-redef, misc]
            pass

        class ModelB(base_type):  # type: ignore[no-redef, misc]
            pass

    schema = get_schema_for_field_definition(
        FieldDefinition.from_kwarg(name="Lookup", annotation=Union[ModelA, ModelB])
    )
    assert schema.one_of == [
        Reference(ref="#/components/schemas/tests_unit_test_openapi_test_schema_test_type_union.ModelA"),
        Reference(ref="#/components/schemas/tests_unit_test_openapi_test_schema_test_type_union.ModelB"),
    ]


@pytest.mark.parametrize("base_type", [msgspec.Struct, TypedDict, dataclass])
def test_type_union_with_none(base_type: type) -> None:
    # https://github.com/litestar-org/litestar/issues/2971
    if base_type is dataclass:

        @dataclass
        class ModelA:  # pyright: ignore
            pass

        @dataclass
        class ModelB:  # pyright: ignore
            pass

    else:

        class ModelA(base_type):  # type: ignore[no-redef, misc]
            pass

        class ModelB(base_type):  # type: ignore[no-redef, misc]
            pass

    schema = get_schema_for_field_definition(
        FieldDefinition.from_kwarg(name="Lookup", annotation=Union[ModelA, ModelB, None])
    )
    assert schema.one_of == [
        Reference(ref="#/components/schemas/tests_unit_test_openapi_test_schema_test_type_union_with_none.ModelA"),
        Reference("#/components/schemas/tests_unit_test_openapi_test_schema_test_type_union_with_none.ModelB"),
        Schema(type=OpenAPIType.NULL),
    ]


def test_default_only_on_field_definition() -> None:
    field_definition = FieldDefinition.from_annotation(int, default=10)
    assert field_definition.kwarg_definition is None

    schema = get_schema_for_field_definition(field_definition)
    assert schema.default == 10


def test_default_not_provided_for_kwarg_but_for_field() -> None:
    field_definition = FieldDefinition.from_annotation(int, default=10, kwarg_definition=KwargDefinition())
    schema = get_schema_for_field_definition(field_definition)

    assert schema.default == 10


def test_routes_with_different_path_param_types_get_merged() -> None:
    # https://github.com/litestar-org/litestar/issues/2700
    @get("/{param:int}")
    async def get_handler(param: int) -> None:
        pass

    @post("/{param:str}")
    async def post_handler(param: str) -> None:
        pass

    app = Litestar([get_handler, post_handler])
    assert app.openapi_schema.paths
    paths = app.openapi_schema.paths["/{param}"]
    assert paths.get is not None
    assert paths.post is not None


def test_unconsumed_path_parameters_are_documented() -> None:
    # https://github.com/litestar-org/litestar/issues/3290
    # https://github.com/litestar-org/litestar/issues/3369

    async def dd(param3: Annotated[str, Parameter(description="123")]) -> str:
        return param3

    async def d(dep_dep: str, param2: Annotated[str, Parameter(description="abc")]) -> str:
        return f"{dep_dep}_{param2}"

    @get("/{param1:str}/{param2:str}/{param3:str}", dependencies={"dep": d, "dep_dep": dd})
    async def handler(dep: str) -> None:
        pass

    app = Litestar([handler])
    params = app.openapi_schema.paths["/{param1}/{param2}/{param3}"].get.parameters  # type: ignore[index, union-attr]
    assert params
    assert len(params) == 3
    for i, param in enumerate(sorted(params, key=lambda p: p.name), 1):  # pyright: ignore
        assert isinstance(param, OpenAPIParameter)
        assert param.name == f"param{i}"
        assert param.required is True
        assert param.param_in is ParamType.PATH


def test_type_alias_type() -> None:
    @get("/")
    def handler(query_param: Annotated[TypeAliasType("IntAlias", int), Parameter(description="foo")]) -> None:  # type: ignore[valid-type]
        pass

    app = Litestar([handler])
    param = app.openapi_schema.paths["/"].get.parameters[0]  # type: ignore[index, union-attr]
    assert param.schema.type is OpenAPIType.INTEGER  # type: ignore[union-attr]
    # ensure other attributes than the plain type are carried over correctly
    assert param.description == "foo"


@pytest.mark.skipif(sys.version_info < (3, 12), reason="type keyword not available before 3.12")
def test_type_alias_type_keyword() -> None:
    ctx: Dict[str, Any] = {}
    exec("type IntAlias = int", ctx, None)
    annotation = ctx["IntAlias"]

    @get("/")
    def handler(query_param: Annotated[annotation, Parameter(description="foo")]) -> None:  # type: ignore[valid-type]
        pass

    app = Litestar([handler])
    param = app.openapi_schema.paths["/"].get.parameters[0]  # type: ignore[union-attr, index]
    assert param.schema.type is OpenAPIType.INTEGER  # type: ignore[union-attr]
    # ensure other attributes than the plain type are carried over correctly
    assert param.description == "foo"



from asyncio import sleep
from typing import AsyncGenerator

from litestar import Litestar, get
from litestar.response import ServerSentEvent, ServerSentEventMessage
from litestar.types import SSEData


async def my_generator() -> AsyncGenerator[SSEData, None]:
    count = 0
    while count < 10:
        await sleep(0.01)
        count += 1
        # In the generator you can yield integers, strings, bytes, dictionaries, or ServerSentEventMessage objects
        # dicts can have the following keys: data, event, id, retry, comment

        # here we yield an integer
        yield count
        # here a string
        yield str(count)
        # here bytes
        yield str(count).encode("utf-8")
        # here a dictionary
        yield {"data": 2 * count, "event": "event2", "retry": 10}
        # here a ServerSentEventMessage object
        yield ServerSentEventMessage(event="something-with-comment", retry=1000, comment="some comment")


@get(path="/count", sync_to_thread=False)
def sse_handler() -> ServerSentEvent:
    return ServerSentEvent(my_generator())


app = Litestar(route_handlers=[sse_handler])



from __future__ import annotations

from uuid import UUID

from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column, relationship
from typing_extensions import Annotated

from litestar import Litestar, put
from litestar.dto import DTOConfig
from litestar.plugins.sqlalchemy import SQLAlchemyDTO

from .my_lib import Base


class A(Base):
    b_id: Mapped[UUID] = mapped_column(ForeignKey("b.id"))
    b: Mapped[B] = relationship(back_populates="a")


class B(Base):
    a: Mapped[A] = relationship(back_populates="b")


data_config = DTOConfig(max_nested_depth=0)
DataDTO = SQLAlchemyDTO[Annotated[A, data_config]]

# default config sets max_nested_depth to 1
ReturnDTO = SQLAlchemyDTO[A]


@put("/a", dto=DataDTO, return_dto=ReturnDTO, sync_to_thread=False)
def update_a(data: A) -> A:
    # this shows that "b" was not parsed out of the inbound data
    assert "b" not in vars(data)
    # Now we'll create an instance of B and assign it"
    # This includes a reference back to ``a`` which is not serialized in the return data
    # because default ``max_nested_depth`` is set to 1
    data.b = B(id=data.b_id, a=data)
    return data


app = Litestar(route_handlers=[update_a])

# run: /a -H "Content-Type: application/json" -X PUT -d '{"id": "6955e63c-c2bc-4707-8fa4-2144d1764746", "b_id": "9cf3518d-7e19-4215-9ec2-e056cac55bf7", "b": {"id": "9cf3518d-7e19-4215-9ec2-e056cac55bf7"}}'



from __future__ import annotations

from http import HTTPStatus
from unittest.mock import MagicMock

import pytest

from litestar import Litestar, asgi
from litestar.config.cors import CORSConfig
from litestar.enums import ScopeType
from litestar.testing import TestClient
from litestar.types.asgi_types import ASGIApp, Receive, Scope, Send


@pytest.fixture(name="asgi_mock")
def asgi_mock_fixture() -> MagicMock:
    return MagicMock()


@pytest.fixture(name="asgi_app")
def asgi_app_fixture(asgi_mock: MagicMock) -> ASGIApp:
    async def asgi_app(scope: Scope, receive: Receive, send: Send) -> None:
        asgi_mock()

        assert scope["type"] == ScopeType.HTTP

        while True:
            event = await receive()
            if event["type"] == "http.request" and not event.get("more_body", False):
                break

        await send(
            {
                "type": "http.response.start",
                "status": 200,
                "headers": [
                    (b"content-type", b"text/plain"),
                ],
            }
        )

        await send(
            {
                "type": "http.response.body",
                "body": b"Hello, world!",
                "more_body": False,
            }
        )

    return asgi_app


def test_cors_middleware_for_mount(asgi_app: ASGIApp, asgi_mock: MagicMock) -> None:
    cors_config = CORSConfig(allow_methods=["*"], allow_origins=["https://some-domain.com"])
    app = Litestar(
        cors_config=cors_config,
        route_handlers=[
            asgi("/app", is_mount=True)(asgi_app),
        ],
        openapi_config=None,
    )

    with TestClient(app) as client:
        response = client.options(
            "http://127.0.0.1:8000/app",
            headers={"origin": "https://some-domain.com"},
        )
        assert response.status_code == HTTPStatus.NO_CONTENT
        assert response.headers["access-control-allow-origin"] == "https://some-domain.com"
    asgi_mock.assert_not_called()


def test_asgi_app_no_origin_header(asgi_app: ASGIApp, asgi_mock: MagicMock) -> None:
    cors_config = CORSConfig(allow_methods=["*"], allow_origins=["https://some-domain.com"])
    app = Litestar(
        cors_config=cors_config,
        route_handlers=[
            asgi("/app", is_mount=True)(asgi_app),
        ],
        openapi_config=None,
    )

    with TestClient(app) as client:
        response = client.options("http://127.0.0.1/app")
        assert response.status_code == HTTPStatus.OK
        assert response.headers["content-type"] == "text/plain"
        asgi_mock.assert_called()


def test_asgi_app_without_cors_configuration(asgi_app: ASGIApp, asgi_mock: MagicMock) -> None:
    non_cors_app = Litestar(
        route_handlers=[asgi("/app", is_mount=True)(asgi_app)],
        openapi_config=None,
    )

    with TestClient(non_cors_app) as client:
        response = client.options("http://127.0.0.1:8000/app")
        assert response.status_code == HTTPStatus.OK
        assert response.headers["content-type"] == "text/plain"
        asgi_mock.assert_called()



from litestar import Litestar
from litestar.contrib.mako import MakoTemplateEngine
from litestar.middleware.session.server_side import ServerSideSessionConfig
from litestar.plugins.flash import FlashConfig, FlashPlugin
from litestar.template.config import TemplateConfig

template_config = TemplateConfig(engine=MakoTemplateEngine, directory="templates")
flash_plugin = FlashPlugin(config=FlashConfig(template_config=template_config))

app = Litestar(
    plugins=[flash_plugin],
    middleware=[ServerSideSessionConfig().middleware],
)



# ruff: noqa: F401
# pyright: reportUnusedImport=false
"""Application ORM configuration."""

from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.utils import warn_deprecation

__all__ = (
    "AuditColumns",
    "BigIntAuditBase",
    "BigIntBase",
    "BigIntPrimaryKey",
    "CommonTableAttributes",
    "ModelProtocol",
    "UUIDAuditBase",
    "UUIDBase",
    "UUIDPrimaryKey",
    "create_registry",
    "orm_registry",
    "touch_updated_timestamp",
)


def __getattr__(attr_name: str) -> object:
    if attr_name in __all__:
        if attr_name == "touch_updated_timestamp":
            try:
                # v0.6.0+
                from advanced_alchemy._listeners import touch_updated_timestamp  # pyright: ignore
            except ImportError:
                from advanced_alchemy.base import touch_updated_timestamp  # type: ignore[no-redef,attr-defined]
            warn_deprecation(
                deprecated_name=f"litestar.contrib.sqlalchemy.base.{attr_name}",
                version="2.12",
                kind="import",
                removal_in="3.0",
                info=f"importing {attr_name} from 'litestar.contrib.sqlalchemy.base' is deprecated, please"
                f"see the 'Advanced Alchemy' documentation for more details on how to use '{attr_name}' instead",
            )
            value = globals()[attr_name] = locals()[attr_name]  # pyright: ignore[reportUnknownVariableType]
            return value  # pyright: ignore[reportUnknownVariableType]
        from advanced_alchemy.base import (  # pyright: ignore[reportMissingImports]
            BigIntAuditBase,
            BigIntBase,
            CommonTableAttributes,
            ModelProtocol,
            UUIDAuditBase,
            UUIDBase,
            create_registry,
            orm_registry,
        )
        from advanced_alchemy.mixins import (  # pyright: ignore[reportMissingImports]
            AuditColumns,
            BigIntPrimaryKey,
            UUIDPrimaryKey,
        )

        warn_deprecation(
            deprecated_name=f"litestar.contrib.sqlalchemy.base.{attr_name}",
            version="2.12",
            kind="import",
            removal_in="3.0",
            info=f"importing {attr_name} from 'litestar.contrib.sqlalchemy.base' is deprecated, please"
            f"import it from 'litestar.plugins.sqlalchemy.base.{attr_name}' instead",
        )
        value = globals()[attr_name] = locals()[attr_name]  # pyright: ignore[reportUnknownVariableType]
        return value

    raise AttributeError(f"module {__name__!r} has no attribute {attr_name!r}")  # pragma: no cover


if TYPE_CHECKING:
    try:
        # v0.6.0+
        from advanced_alchemy._listeners import touch_updated_timestamp  # pyright: ignore
    except ImportError:
        from advanced_alchemy.base import touch_updated_timestamp  # type: ignore[no-redef,attr-defined]

    from advanced_alchemy.base import (  # pyright: ignore[reportMissingImports]
        BigIntAuditBase,
        BigIntBase,
        CommonTableAttributes,
        ModelProtocol,
        UUIDAuditBase,
        UUIDBase,
        create_registry,
        orm_registry,
    )
    from advanced_alchemy.mixins import (  # pyright: ignore[reportMissingImports]
        AuditColumns,
        BigIntPrimaryKey,
        UUIDPrimaryKey,
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Sequence

from sqlalchemy import select
from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column

from litestar import Litestar, post
from litestar.plugins.sqlalchemy import SQLAlchemyPlugin, SQLAlchemySyncConfig

if TYPE_CHECKING:
    from sqlalchemy.orm import Session


class Base(DeclarativeBase): ...


class TodoItem(Base):
    __tablename__ = "todo_item"
    title: Mapped[str] = mapped_column(primary_key=True)
    done: Mapped[bool]


@post("/", sync_to_thread=True)
def add_item(data: TodoItem, db_session: Session) -> Sequence[TodoItem]:
    with db_session.begin():
        db_session.add(data)
    return db_session.execute(select(TodoItem)).scalars().all()


config = SQLAlchemySyncConfig(connection_string="sqlite:///todo_sync.sqlite", create_all=True, metadata=Base.metadata)
plugin = SQLAlchemyPlugin(config=config)
app = Litestar(route_handlers=[add_item], plugins=[plugin])



from typing import TYPE_CHECKING

from litestar import Litestar

if TYPE_CHECKING:
    from litestar.config.app import AppConfig


async def close_db_connection() -> None:
    """Closes the database connection on application shutdown."""


def receive_app_config(app_config: "AppConfig") -> "AppConfig":
    """Receives parameters from the application.

    In reality, this would be a library of boilerplate that is carried from one application to another, or a third-party
    developed application configuration tool.
    """
    app_config.on_shutdown.append(close_db_connection)
    return app_config


app = Litestar([], on_app_init=[receive_app_config])



from litestar import Litestar, get
from litestar.config.cors import CORSConfig
from litestar.status_codes import HTTP_204_NO_CONTENT
from litestar.testing import TestClient


@get("/credentials-test")
async def credentials_handler() -> str:
    return "Test Successful!"


def test_cors_with_credentials_allowed() -> None:
    cors_config = CORSConfig(
        allow_methods=["GET"], allow_origins=["https://allowed-origin.com"], allow_credentials=True
    )
    app = Litestar(route_handlers=[credentials_handler], cors_config=cors_config)

    with TestClient(app) as client:
        response = client.options(
            "/endpoint", headers={"Origin": "https://allowed-origin.com", "Access-Control-Request-Method": "GET"}
        )
        assert response.status_code == HTTP_204_NO_CONTENT
        assert response.headers["access-control-allow-credentials"] == "true"


def test_cors_with_credentials_disallowed() -> None:
    cors_config = CORSConfig(
        allow_methods=["GET"],
        allow_origins=["https://allowed-origin.com"],
        allow_credentials=False,  # Credentials should not be allowed
    )
    app = Litestar(route_handlers=[credentials_handler], cors_config=cors_config)

    with TestClient(app) as client:
        response = client.options(
            "/endpoint", headers={"Origin": "https://allowed-origin.com", "Access-Control-Request-Method": "GET"}
        )
        assert response.status_code == HTTP_204_NO_CONTENT
        assert "access-control-allow-credentials" not in response.headers



from __future__ import annotations

from litestar import Litestar

from .controller import MyController
from .domain import Model

app = Litestar(route_handlers=[MyController], signature_types=[Model])



from pathlib import Path

from litestar import Litestar
from litestar.config.response_cache import ResponseCacheConfig
from litestar.middleware.rate_limit import RateLimitConfig
from litestar.middleware.session.server_side import ServerSideSessionConfig
from litestar.stores.file import FileStore
from litestar.stores.redis import RedisStore

app = Litestar(
    stores={"redis": RedisStore.with_client(), "file": FileStore(Path("data"))},
    response_cache_config=ResponseCacheConfig(store="redis"),
    middleware=[
        ServerSideSessionConfig(store="file").middleware,
        RateLimitConfig(rate_limit=("second", 10), store="redis").middleware,
    ],
)



from __future__ import annotations

from litestar.handlers.websocket_handlers.listener import (
    WebsocketListener,
    WebsocketListenerRouteHandler,
    websocket_listener,
)
from litestar.handlers.websocket_handlers.route_handler import WebsocketRouteHandler, websocket
from litestar.handlers.websocket_handlers.stream import send_websocket_stream, websocket_stream

__all__ = (
    "WebsocketListener",
    "WebsocketListenerRouteHandler",
    "WebsocketRouteHandler",
    "send_websocket_stream",
    "websocket",
    "websocket_listener",
    "websocket_stream",
)



from litestar.stores.memory import MemoryStore

store = MemoryStore()


async def main() -> None:
    value = await store.get("key")
    print(value)  # this will print 'None', as no store with this key has been defined yet

    await store.set("key", b"value")
    value = await store.get("key")
    print(value)



from litestar import Litestar
from litestar.static_files import create_static_files_router

app = Litestar(
    route_handlers=[
        create_static_files_router(
            path="/static",
            directories=["assets"],
            send_as_attachment=True,
        )
    ]
)



from __future__ import annotations

from typing import Any

from litestar.exceptions import responses
from litestar.middleware._internal.exceptions import middleware
from litestar.utils.deprecation import warn_deprecation


def __getattr__(name: str) -> Any:
    if name == "ExceptionHandlerMiddleware":
        warn_deprecation(
            version="2.9",
            deprecated_name=name,
            kind="class",
            removal_in="3.0",
            info="ExceptionHandlerMiddleware has been removed from the public API.",
        )
        return middleware.ExceptionHandlerMiddleware

    if name == "create_exception_response":
        warn_deprecation(
            version="2.9",
            deprecated_name=name,
            kind="function",
            removal_in="3.0",
            alternative="litestar.exceptions.responses.create_exception_response",
        )
        return responses.create_exception_response

    if name == "ExceptionResponseContent":
        warn_deprecation(
            version="2.9",
            deprecated_name=name,
            kind="class",
            removal_in="3.0",
            alternative="litestar.exceptions.responses.ExceptionResponseContent",
        )
        return responses.ExceptionResponseContent

    raise AttributeError(f"module {__name__} has no attribute {name}")



from pathlib import Path

from litestar import Litestar, get
from litestar.static_files.config import StaticFilesConfig


@get("/")
async def handler() -> dict[str, str]:
    return {"hello": "world"}


app = Litestar(
    route_handlers=[],
    static_files_config=[
        StaticFilesConfig(directories=[Path(__file__).parent / "public"], path="/", html_mode=True),
    ],
)


if __name__ == "__main__":
    import uvicorn

    uvicorn.run(app)



from litestar import Litestar
from litestar.config.response_cache import ResponseCacheConfig
from litestar.types import HTTPScope


def custom_cache_response_filter(_: HTTPScope, status_code: int) -> bool:
    # Cache only 2xx responses
    return 200 <= status_code < 300


response_cache_config = ResponseCacheConfig(cache_response_filter=custom_cache_response_filter)

# Create the app with a custom cache response filter
app = Litestar(
    response_cache_config=response_cache_config,
)



from typing import Dict, Optional

from litestar import Litestar, get


@get("/", sync_to_thread=False)
def index(param: Optional[str] = None) -> Dict[str, Optional[str]]:
    return {"param": param}


app = Litestar(route_handlers=[index])


# run: /
# run: /?param=goodbye



from __future__ import annotations

from enum import Enum
from pathlib import Path

import pytest

from litestar import Litestar, Request, get, post
from litestar.contrib.jinja import JinjaTemplateEngine
from litestar.contrib.mako import MakoTemplateEngine
from litestar.contrib.minijinja import MiniJinjaTemplateEngine
from litestar.exceptions import ImproperlyConfiguredException
from litestar.middleware.rate_limit import RateLimitConfig
from litestar.middleware.session.server_side import ServerSideSessionConfig
from litestar.plugins.flash import FlashConfig, FlashPlugin, flash
from litestar.response import Redirect, Template
from litestar.template import TemplateConfig, TemplateEngineProtocol
from litestar.testing import create_test_client

text_html_jinja = """{% for message in get_flashes() %}<span class="{{ message.category }}">{{ message.message }}</span>{% endfor %}"""
text_html_mako = """<% messages = get_flashes() %>\\
% for m in messages:
<span class="${m['category']}">${m['message']}</span>\\
% endfor
"""


class CustomCategory(str, Enum):
    custom1 = "1"
    custom2 = "2"
    custom3 = "3"


class FlashCategory(str, Enum):
    info = "INFO"
    error = "ERROR"
    warning = "WARNING"
    success = "SUCCESS"


@pytest.mark.parametrize(
    "engine, template_str",
    (
        (JinjaTemplateEngine, text_html_jinja),
        (MakoTemplateEngine, text_html_mako),
        (MiniJinjaTemplateEngine, text_html_jinja),
    ),
    ids=("jinja", "mako", "minijinja"),
)
@pytest.mark.parametrize(
    "category_enum",
    (CustomCategory, FlashCategory),
    ids=("custom_category", "flash_category"),
)
def test_flash_plugin(
    tmp_path: Path,
    engine: type[TemplateEngineProtocol],
    template_str: str,
    category_enum: Enum,
) -> None:
    Path(tmp_path / "flash.html").write_text(template_str)

    @get("/")
    async def index() -> Redirect:
        return Redirect("/login")

    @get("/login")
    async def login(request: Request) -> Template:
        flash(request, "Flash Test!", category="info")
        return Template("flash.html")

    @post("/check")
    async def check(request: Request) -> Redirect:
        flash(request, "User not Found!", category="warning")
        return Redirect("/login")

    template_config: TemplateConfig = TemplateConfig(
        directory=Path(tmp_path),
        engine=engine,
    )
    session_config = ServerSideSessionConfig()
    flash_config = FlashConfig(template_config=template_config)
    with create_test_client(
        plugins=[FlashPlugin(config=flash_config)],
        route_handlers=[index, login, check],
        template_config=template_config,
        middleware=[session_config.middleware],
    ) as client:
        r = client.get("/")
        assert r.status_code == 200
        assert "Flash Test!" in r.text
        r = client.get("/login")
        assert r.status_code == 200
        assert "Flash Test!" in r.text
        r = client.post("/check")
        assert r.status_code == 200
        assert "User not Found!" in r.text
        assert "Flash Test!" in r.text


def test_flash_config_doesnt_have_session() -> None:
    template_config = TemplateConfig(directory=Path("tests/templates"), engine=JinjaTemplateEngine)
    flash_config = FlashConfig(template_config=template_config)
    with pytest.raises(ImproperlyConfiguredException):
        Litestar(plugins=[FlashPlugin(config=flash_config)])


def test_flash_config_has_wrong_middleware_type() -> None:
    template_config = TemplateConfig(directory=Path("tests/templates"), engine=JinjaTemplateEngine)
    flash_config = FlashConfig(template_config=template_config)
    rate_limit_config = RateLimitConfig(rate_limit=("minute", 1), exclude=["/schema"])
    with pytest.raises(ImproperlyConfiguredException):
        Litestar(plugins=[FlashPlugin(config=flash_config)], middleware=[rate_limit_config.middleware])



from asyncio import sleep

from litestar.stores.memory import MemoryStore

store = MemoryStore()


async def main() -> None:
    await store.set("foo", b"bar", expires_in=1)
    await sleep(0.5)

    await store.get("foo", renew_for=1)  # this will reset the time to live to one second

    await sleep(1)
    # it has now been 1.5 seconds since the key was set with a life time of one second,
    # so it should have expired however, since it was renewed for one second, it is still available
    value = await store.get("foo")
    print(value)



import logging
from typing import Dict

from litestar import Litestar, get
from litestar.background_tasks import BackgroundTask

logger = logging.getLogger(__name__)


async def logging_task(identifier: str, message: str) -> None:
    logger.info("%s: %s", identifier, message)


@get("/", background=BackgroundTask(logging_task, "greeter", message="was called"), sync_to_thread=False)
def greeter() -> Dict[str, str]:
    return {"hello": "world"}


app = Litestar(route_handlers=[greeter])



from pathlib import Path

import pytest
from pytest import MonkeyPatch
from sqlalchemy.ext.asyncio import create_async_engine
from sqlalchemy.pool import NullPool

from litestar.plugins.sqlalchemy import AsyncSessionConfig, SQLAlchemyAsyncConfig
from litestar.testing import TestClient

pytestmark = pytest.mark.xdist_group("sqlalchemy_examples")


async def test_sqlalchemy_declarative_models(tmp_path: Path, monkeypatch: MonkeyPatch) -> None:
    engine = create_async_engine("sqlite+aiosqlite:///test.sqlite", poolclass=NullPool)

    session_config = AsyncSessionConfig(expire_on_commit=False)
    sqlalchemy_config = SQLAlchemyAsyncConfig(
        session_config=session_config,
        create_all=True,
        engine_instance=engine,
    )  # Create 'async_session' dependency.
    from docs.examples.contrib.sqlalchemy import sqlalchemy_declarative_models

    monkeypatch.setattr(sqlalchemy_declarative_models, "sqlalchemy_config", sqlalchemy_config)
    async with engine.begin() as connection:
        await connection.run_sync(sqlalchemy_declarative_models.Author.metadata.create_all)
        await connection.commit()
    with TestClient(sqlalchemy_declarative_models.app) as client:
        response = client.get("/authors")
        assert response.status_code == 200
        assert len(response.json()) > 0



import logging
from typing import TYPE_CHECKING

from litestar import Litestar, get
from litestar.exceptions import HTTPException
from litestar.status_codes import HTTP_400_BAD_REQUEST

logger = logging.getLogger()

if TYPE_CHECKING:
    from litestar.types import Scope


@get("/some-path", sync_to_thread=False)
def my_handler() -> None:
    """Route handler that raises an exception."""
    raise HTTPException(detail="bad request", status_code=HTTP_400_BAD_REQUEST)


async def after_exception_handler(exc: Exception, scope: "Scope") -> None:
    """Hook function that will be invoked after each exception."""
    state = Litestar.from_scope(scope).state
    if not hasattr(state, "error_count"):
        state.error_count = 1
    else:
        state.error_count += 1

    logger.info(
        "an exception of type %s has occurred for requested path %s and the application error count is %d.",
        type(exc).__name__,
        scope["path"],
        state.error_count,
    )


app = Litestar([my_handler], after_exception=[after_exception_handler])



from sqlalchemy.orm import Mapped

from litestar import Litestar, websocket_listener
from litestar.plugins.sqlalchemy import SQLAlchemyDTO, base


class User(base.UUIDBase):
    name: Mapped[str]


UserDTO = SQLAlchemyDTO[User]


@websocket_listener("/", dto=UserDTO)
async def handler(data: User) -> User:
    return data


app = Litestar([handler])



from litestar import Litestar
from litestar.stores.memory import MemoryStore
from litestar.stores.registry import StoreRegistry

memory_store = MemoryStore()


def default_factory(name: str) -> MemoryStore:
    return memory_store


app = Litestar([], stores=StoreRegistry(default_factory=default_factory))



from dataclasses import dataclass
from typing import List, Optional

from litestar import Litestar, get, post, put
from litestar.exceptions import NotFoundException


@dataclass
class TodoItem:
    title: str
    done: bool


TODO_LIST: List[TodoItem] = [
    TodoItem(title="Start writing TODO list", done=True),
    TodoItem(title="???", done=False),
    TodoItem(title="Profit", done=False),
]


def get_todo_by_title(todo_name) -> TodoItem:
    for item in TODO_LIST:
        if item.title == todo_name:
            return item
    raise NotFoundException(detail=f"TODO {todo_name!r} not found")


@get("/")
async def get_list(done: Optional[bool] = None) -> List[TodoItem]:
    if done is None:
        return TODO_LIST
    return [item for item in TODO_LIST if item.done == done]


@post("/")
async def add_item(data: TodoItem) -> List[TodoItem]:
    TODO_LIST.append(data)
    return TODO_LIST


@put("/{item_title:str}")
async def update_item(item_title: str, data: TodoItem) -> List[TodoItem]:
    todo_item = get_todo_by_title(item_title)
    todo_item.title = data.title
    todo_item.done = data.done
    return TODO_LIST


app = Litestar([get_list, add_item, update_item])



from uuid import UUID, uuid4

from pydantic import BaseModel

from litestar import Litestar, get


class IdModel(BaseModel):
    __schema_name__ = "IdContainer"

    id: UUID


@get("/id", sync_to_thread=False)
def retrieve_id_handler() -> IdModel:
    """

    Returns: An IdModel

    """
    return IdModel(id=uuid4())


app = Litestar(route_handlers=[retrieve_id_handler])



from datetime import datetime

from sqlalchemy.orm import Mapped, mapped_column

from litestar import Litestar, post
from litestar.dto import dto_field
from litestar.plugins.sqlalchemy import SQLAlchemyDTO

from .my_lib import Base


class User(Base):
    # `Base` defines `id` field as:
    # id: Mapped[UUID] = mapped_column(default=uuid4, primary_key=True)
    name: Mapped[str]
    password: Mapped[str] = mapped_column(info=dto_field("private"))
    created_at: Mapped[datetime] = mapped_column(info=dto_field("read-only"))


UserDTO = SQLAlchemyDTO[User]


@post("/users", dto=UserDTO, sync_to_thread=False)
def create_user(data: User) -> User:
    # even though the client did not send the id field,
    # since it is a primary key it is autogenerated
    assert "id" in vars(data)
    # even though the client sent the password and created_at field, it is not in the data object
    assert "password" not in vars(data)
    assert "created_at" not in vars(data)
    # normally the database would set the created_at timestamp
    data.created_at = datetime.min
    return data  # the response includes the created_at field


app = Litestar(route_handlers=[create_user])

# run: /users -H "Content-Type: application/json" -d '{"name":"Litestar User","password":"xyz","created_at":"2023-04-24T00:00:00Z"}'



# ruff: noqa: F401
from __future__ import annotations

import sys
import warnings
from importlib.util import cache_from_source
from pathlib import Path

import pytest

from litestar.contrib import attrs as contrib_attrs
from litestar.plugins import attrs as plugin_attrs


def purge_module(module_names: list[str], path: str | Path) -> None:
    for name in module_names:
        if name in sys.modules:
            del sys.modules[name]
    Path(cache_from_source(str(path))).unlink(missing_ok=True)


def test_contrib_attrs_deprecation_warning() -> None:
    """Test that importing from contrib.attrs raises a deprecation warning."""
    purge_module(["litestar.contrib.attrs"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing AttrsSchemaPlugin from 'litestar.contrib.attrs' is deprecated"
    ):
        from litestar.contrib.attrs import AttrsSchemaPlugin


def test_contrib_attrs_schema_deprecation_warning() -> None:
    """Test that importing from contrib.attrs raises a deprecation warning."""
    purge_module(["litestar.contrib.attrs.attrs_schema_plugin"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing AttrsSchemaPlugin from 'litestar.contrib.attrs.attrs_schema_plugin' is deprecated",
    ):
        from litestar.contrib.attrs.attrs_schema_plugin import AttrsSchemaPlugin


def test_functionality_parity() -> None:
    """Test that the functionality is identical between contrib and plugin versions."""
    assert contrib_attrs.AttrsSchemaPlugin is plugin_attrs.AttrsSchemaPlugin
    assert contrib_attrs.is_attrs_class is plugin_attrs.is_attrs_class



from litestar.openapi.plugins import RedocRenderPlugin

redoc_plugin = RedocRenderPlugin(
    version="latest",
    google_fonts=True,
    path="/redoc",
    js_url="https://cdn.company.internal/redoc/custom-redoc.js",
)



from .emitter import BaseEventEmitterBackend, SimpleEventEmitter
from .listener import EventListener, listener

__all__ = ("BaseEventEmitterBackend", "EventListener", "SimpleEventEmitter", "listener")



from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.utils import warn_deprecation

if TYPE_CHECKING:
    from litestar_htmx._utils import (
        HTMXHeaders,
        get_headers,
        get_location_headers,
        get_push_url_header,
        get_redirect_header,
        get_refresh_header,
        get_replace_url_header,
        get_reswap_header,
        get_retarget_header,
        get_trigger_event_headers,
    )
__all__ = (
    "HTMXHeaders",
    "get_headers",
    "get_location_headers",
    "get_push_url_header",
    "get_redirect_header",
    "get_refresh_header",
    "get_replace_url_header",
    "get_reswap_header",
    "get_retarget_header",
    "get_trigger_event_headers",
)


def __getattr__(attr_name: str) -> object:
    if attr_name in __all__:
        from litestar_htmx import _utils as utils

        module = "litestar.plugins.htmx._utils"
        value = globals()[attr_name] = getattr(utils, attr_name)

        warn_deprecation(
            deprecated_name=f"litestar.contrib.htmx._utils.{attr_name}",
            version="2.13",
            kind="import",
            removal_in="3.0",
            info=f"importing {attr_name} from 'litestar.contrib.htmx._utils' is deprecated, please import it from '{module}' instead",
        )

        return value

    raise AttributeError(f"module {__name__!r} has no attribute {attr_name!r}")



from pathlib import Path
from typing import Any, Dict

from litestar import Litestar, get
from litestar.contrib.mako import MakoTemplateEngine
from litestar.response import Template
from litestar.template.config import TemplateConfig


def my_template_function(ctx: Dict[str, Any]) -> str:
    return ctx.get("my_context_key", "nope")


def register_template_callables(engine: MakoTemplateEngine) -> None:
    engine.register_template_callable(
        key="check_context_key",
        template_callable=my_template_function,
    )


template_config = TemplateConfig(
    directory=Path(__file__).parent / "templates",
    engine=MakoTemplateEngine,
    engine_callback=register_template_callables,
)


@get("/", sync_to_thread=False)
def index() -> Template:
    return Template(template_name="index.html.mako")


app = Litestar(
    route_handlers=[index],
    template_config=template_config,
)



from __future__ import annotations

from dataclasses import dataclass

from litestar import Litestar, post
from litestar.dto import DataclassDTO, DTOConfig, DTOData


@dataclass
class Address:
    id: int
    street: str


@dataclass
class Person:
    id: int
    name: str
    age: int
    address: Address


class ReadDTO(DataclassDTO[Person]):
    config = DTOConfig()


class WriteDTO(DataclassDTO[Person]):
    config = DTOConfig(exclude={"id", "address.id"})


@post("/person", dto=WriteDTO, return_dto=ReadDTO, sync_to_thread=False)
def create_person(data: DTOData[Person]) -> Person:
    # Logic for persisting the person goes here
    return data.create_instance(id=1, address__id=2)


app = Litestar(route_handlers=[create_person])

# run: /person -H "Content-Type: application/json" -d '{"name":"Peter","age":41, "address": {"street": "Fake Street"}}'



from __future__ import annotations

import importlib
import logging
import multiprocessing
import os
import re
import shlex
import socket
import subprocess
import sys
import time
from contextlib import contextmanager, redirect_stderr
from pathlib import Path
from typing import TYPE_CHECKING, Generator

import httpx
import uvicorn
from auto_pytabs.sphinx_ext import CodeBlockOverride, LiteralIncludeOverride
from docutils.nodes import Node, admonition, literal_block, title
from docutils.parsers.rst import directives
from sphinx.addnodes import highlightlang

from litestar import Litestar

if TYPE_CHECKING:
    from sphinx.application import Sphinx


RGX_RUN = re.compile(r"# +?run:(.*)")


logger = logging.getLogger("sphinx")

ignore_missing_output = os.getenv("LITESTAR_DOCS_IGNORE_MISSING_EXAMPLE_OUTPUT", "") == "1"


class StartupError(RuntimeError):
    pass


def _load_app_from_path(path: Path) -> Litestar:
    module = importlib.import_module(str(path.with_suffix("")).replace("/", "."))
    for obj in module.__dict__.values():
        if isinstance(obj, Litestar):
            return obj
    raise RuntimeError(f"No Litestar app found in {path}")


def _get_available_port() -> int:
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        # Bind to a free port provided by the host
        try:
            sock.bind(("localhost", 0))
        except OSError as e:
            raise StartupError("Could not find an open port") from e
        else:
            return sock.getsockname()[1]


@contextmanager
def run_app(path: Path) -> Generator[int, None, None]:
    """Run an example app from a python file.

    The first ``Litestar`` instance found in the file will be used as target to run.
    """

    port = _get_available_port()
    app = _load_app_from_path(path)

    def run() -> None:
        with redirect_stderr(Path(os.devnull).open()):
            uvicorn.run(app, port=port, access_log=False)

    count = 0
    while count < 100:
        proc = multiprocessing.Process(target=run)
        proc.start()
        try:
            for _ in range(100):
                try:
                    httpx.get(f"http://127.0.0.1:{port}", timeout=0.1)
                    break
                except httpx.TransportError:
                    time.sleep(0.1)
            else:
                raise StartupError(f"App {path} failed to come online")

            yield port
            break
        except StartupError:
            time.sleep(0.2)
            count += 1
            port = _get_available_port()
        finally:
            proc.kill()

    else:
        raise StartupError(f"App {path} failed to come online")


def extract_run_args(content: str) -> tuple[str, list[list[str]]]:
    """Extract run args from a python file.

    Return the file content stripped of the run comments and a list of argument lists
    """
    new_lines = []
    run_configs = []
    for line in content.splitlines():
        if run_stmt_match := RGX_RUN.match(line):
            run_stmt = run_stmt_match.group(1).lstrip()
            run_configs.append(shlex.split(run_stmt))
        else:
            new_lines.append(line)
    return "\n".join(new_lines), run_configs


def exec_examples(app_file: Path, run_configs: list[list[str]]) -> str:
    """Start a server with the example application, run the specified requests against it
    and return their results
    """

    results = []

    with run_app(app_file) as port:
        for run_args in run_configs:
            url_path, *options = run_args
            args = ["curl", "-s", f"http://127.0.0.1:{port}{url_path}", *options]
            clean_args = ["curl", f"http://127.0.0.1:8000{url_path}", *options]

            proc = subprocess.run(  # noqa: PLW1510, S603
                args,
                capture_output=True,
                text=True,
            )
            stdout = proc.stdout.splitlines()
            if not stdout:
                logger.debug(proc.stderr)
                if not ignore_missing_output:
                    logger.error(f"Example: {app_file}:{args} yielded no results")
                continue

            result = "\n".join(("> " + (" ".join(clean_args)), *stdout))
            results.append(result)

    return "\n".join(results)


class LiteralInclude(LiteralIncludeOverride):
    option_spec = {**LiteralIncludeOverride.option_spec, "no-run": directives.flag}

    def run(self) -> list[Node]:
        cwd = Path.cwd()
        docs_dir = cwd / "docs"
        language = self.options.get("language")
        file_path = Path(self.env.relfn2path(self.arguments[0])[1])

        if (language != "python" and file_path.suffix != ".py") or "no-run" in self.options:
            return super().run()

        content = file_path.read_text()
        clean_content, run_args = extract_run_args(content)

        if not run_args:
            return super().run()

        tmp_file = self.env.tmp_examples_path / str(file_path.relative_to(docs_dir)).replace("/", "_")

        self.arguments[0] = f"/{tmp_file.relative_to(docs_dir)!s}"
        tmp_file.write_text(clean_content)

        nodes = super().run()

        result = exec_examples(file_path.relative_to(cwd), run_args)

        nodes.append(
            admonition(
                "",
                title("", "Run it"),
                highlightlang(
                    "",
                    literal_block("", result),
                    lang="shell",
                    force=False,
                    linenothreshold=sys.maxsize,
                ),
                literal_block("", result),
            )
        )

        return nodes


def setup(app: Sphinx) -> dict[str, bool]:
    app.add_directive("literalinclude", LiteralInclude, override=True)
    app.add_directive("code-block", CodeBlockOverride, override=True)

    return {"parallel_read_safe": True, "parallel_write_safe": True}



from collections import defaultdict
from typing import Dict

from litestar import Litestar, Request, get

COUNTER: Dict[str, int] = defaultdict(int)


async def after_response(request: Request) -> None:
    COUNTER[request.url.path] += 1


@get("/hello")
async def hello() -> Dict[str, int]:
    return COUNTER


app = Litestar(route_handlers=[hello], after_response=after_response)


# run: /hello
# run: /hello



import os

from piccolo.conf.apps import AppConfig, table_finder

CURRENT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))


APP_CONFIG = AppConfig(
    app_name="home",
    migrations_folder_path=os.path.join(CURRENT_DIRECTORY, "piccolo_migrations"),
    table_classes=table_finder(modules=["home.tables"], exclude_imported=True),
    migration_dependencies=[],
    commands=[],
)



from typing import Dict

from pydantic import BaseModel, ConfigDict
from typing_extensions import Annotated

from litestar import Litestar, post
from litestar.datastructures import UploadFile
from litestar.enums import RequestEncodingType
from litestar.params import Body


class FormData(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)
    cv: UploadFile
    diploma: UploadFile


@post(path="/")
async def handle_file_upload(
    data: Annotated[FormData, Body(media_type=RequestEncodingType.MULTI_PART)],
) -> Dict[str, str]:
    cv_content = await data.cv.read()
    diploma_content = await data.diploma.read()

    return {"cv": cv_content.decode(), "diploma": diploma_content.decode()}


app = Litestar(route_handlers=[handle_file_upload])



"""Test exception handlers defined on layers are called for middleware exceptions."""

from __future__ import annotations

from typing import TYPE_CHECKING
from unittest.mock import MagicMock

from litestar import get
from litestar.connection import Request
from litestar.response import Response
from litestar.testing import create_test_client

if TYPE_CHECKING:
    from litestar.types.asgi_types import ASGIApp, Receive, Scope, Send

exception_handler_mock = MagicMock()
route_handler_mock = MagicMock()


def exception_handler(_: Request, __: type[Exception]) -> Response:
    exception_handler_mock()
    return Response(content="", status_code=500)


def asgi_middleware(app: ASGIApp) -> ASGIApp:
    async def middleware(scope: Scope, receive: Receive, send: Send) -> None:
        raise RuntimeError("damn")

    return middleware


@get("/not_called", exception_handlers={RuntimeError: exception_handler}, sync_to_thread=False)
def not_called() -> None:
    route_handler_mock()


def test_middleware_send_wrapper_called_on_exception() -> None:
    with create_test_client([not_called], middleware=[asgi_middleware], openapi_config=None) as client:
        client.get("/not_called")
        route_handler_mock.assert_not_called()
        exception_handler_mock.assert_called_once()



from litestar import Controller, Litestar, MediaType, get
from litestar.datastructures import Cookie


class MyController(Controller):
    path = "/controller-path"
    response_cookies = [Cookie(key="my-cookie", value="123")]

    @get(
        path="/",
        response_cookies=[Cookie(key="my-cookie", value="456")],
        media_type=MediaType.TEXT,
        sync_to_thread=False,
    )
    def my_route_handler(self) -> str:
        return "hello world"


app = Litestar(route_handlers=[MyController])



from dataclasses import dataclass

from litestar import Litestar, post


@dataclass
class User:
    id: int
    name: str


@post(path="/")
async def index(data: User) -> User:
    return data


app = Litestar(route_handlers=[index])



from random import randint

from pydantic import BaseModel

from litestar import Litestar, Response, Router, get
from litestar.datastructures import Cookie


class Resource(BaseModel):
    id: int
    name: str


@get(
    "/resources",
    response_cookies=[
        Cookie(
            key="Random-Cookie",
            description="a random number in the range 100 - 1000",
            documentation_only=True,
        )
    ],
    sync_to_thread=False,
)
def retrieve_resource() -> Response[Resource]:
    return Response(
        Resource(
            id=1,
            name="my resource",
        ),
        cookies=[Cookie(key="Random-Cookie", value=str(randint(100, 1000)))],
    )


def after_request_handler(response: Response) -> Response:
    response.set_cookie(key="Random-Cookie", value=str(randint(1, 100)))
    return response


router = Router(
    path="/router-path",
    route_handlers=[retrieve_resource],
    after_request=after_request_handler,
    response_cookies=[
        Cookie(
            key="Random-Cookie",
            description="a random number in the range 1 - 100",
            documentation_only=True,
        )
    ],
)

app = Litestar(route_handlers=[router])



import time

from litestar.datastructures import MutableScopeHeaders
from litestar.enums import ScopeType
from litestar.middleware import ASGIMiddleware
from litestar.types import ASGIApp, Message, Receive, Scope, Send


class ProcessTimeHeader(ASGIMiddleware):
    scopes = (ScopeType.HTTP, ScopeType.ASGI)

    async def handle(self, scope: Scope, receive: Receive, send: Send, next_app: ASGIApp) -> None:
        start_time = time.monotonic()

        async def send_wrapper(message: Message) -> None:
            if message["type"] == "http.response.start":
                process_time = time.monotonic() - start_time
                headers = MutableScopeHeaders.from_message(message=message)
                headers["X-Process-Time"] = str(process_time)
            await send(message)

        await next_app(scope, receive, send_wrapper)



from typing import List

from polyfactory.factories.pydantic_factory import ModelFactory
from pydantic import BaseModel

from litestar import Litestar, get
from litestar.pagination import AbstractSyncClassicPaginator, ClassicPagination


class Person(BaseModel):
    id: str
    name: str


class PersonFactory(ModelFactory[Person]):
    __model__ = Person


# we will implement a paginator - the paginator must implement two methods 'get_total' and 'get_items'
# we would usually use a database for this, but for our case we will "fake" the dataset using a factory.


class PersonClassicPaginator(AbstractSyncClassicPaginator[Person]):
    def __init__(self) -> None:
        self.data = PersonFactory.batch(50)

    def get_total(self, page_size: int) -> int:
        return round(len(self.data) / page_size)

    def get_items(self, page_size: int, current_page: int) -> List[Person]:
        return [self.data[i : i + page_size] for i in range(0, len(self.data), page_size)][current_page - 1]


paginator = PersonClassicPaginator()


# we now create a regular handler. The handler will receive two query parameters - 'page_size' and 'current_page', which
# we will pass to the paginator.
@get("/people", sync_to_thread=False)
def people_handler(page_size: int, current_page: int) -> ClassicPagination[Person]:
    return paginator(page_size=page_size, current_page=current_page)


app = Litestar(route_handlers=[people_handler])



from litestar.middleware import ASGIMiddleware
from litestar.types import ASGIApp, Receive, Scope, Send


class MyMiddleware(ASGIMiddleware):
    async def handle(self, scope: Scope, receive: Receive, send: Send, next_app: ASGIApp) -> None:
        # do stuff
        await next_app(scope, receive, send)



from .config import OpenTelemetryConfig
from .middleware import OpenTelemetryInstrumentationMiddleware
from .plugin import OpenTelemetryPlugin

__all__ = (
    "OpenTelemetryConfig",
    "OpenTelemetryInstrumentationMiddleware",
    "OpenTelemetryPlugin",
)



from typing import Any

from litestar import WebSocket, websocket
from litestar.datastructures import State
from litestar.testing import create_test_client


def test_websocket() -> None:
    @websocket(path="/ws")
    async def websocket_handler(socket: WebSocket[Any, Any, State]) -> None:
        await socket.accept()
        recv = await socket.receive_json()
        await socket.send_json({"message": recv})
        await socket.close()

    with create_test_client(route_handlers=[websocket_handler]).websocket_connect("/ws") as ws:
        ws.send_json({"hello": "world"})
        data = ws.receive_json()
        assert data == {"message": {"hello": "world"}}



from http import HTTPStatus

from litestar import Litestar, get
from litestar.config.cors import CORSConfig
from litestar.testing import TestClient


@get("/test")
async def handler() -> str:
    return "Should not reach this"


cors_config = CORSConfig(allow_methods=["GET"], allow_origins=["https://allowed-origin.com"], allow_credentials=True)
app = Litestar(route_handlers=[handler], cors_config=cors_config)


def test_cors_on_middleware_exception_with_origin_header() -> None:
    with TestClient(app) as client:
        response = client.get("/testing", headers={"Origin": "https://allowed-origin.com"})
        assert response.status_code == HTTPStatus.NOT_FOUND
        assert response.headers["access-control-allow-origin"] == "https://allowed-origin.com"
        assert response.headers["access-control-allow-credentials"] == "true"



from concurrent.futures import Future, wait

import anyio

from litestar.testing import create_test_client


def test_with_portal() -> None:
    """This example shows how to manage asynchronous tasks using a portal.

    The test function itself is not async. Asynchronous functions are executed and awaited using the portal.
    """

    async def get_float(value: float) -> float:
        await anyio.sleep(value)
        return value

    with create_test_client(route_handlers=[]) as test_client, test_client.portal() as portal:
        # start a background task with the portal
        future: Future[float] = portal.start_task_soon(get_float, 0.25)
        # do other work
        assert portal.call(get_float, 0.1) == 0.1
        # wait for the background task to complete
        wait([future])
        assert future.done()
        assert future.result() == 0.25



from __future__ import annotations

from typing import TYPE_CHECKING
from unittest.mock import MagicMock

from litestar import Request, Response, asgi
from litestar.testing import create_test_client

if TYPE_CHECKING:
    from litestar.types import Receive, Scope, Send

mock = MagicMock()


def value_error_handler(_: Request, exc: ValueError) -> Response:
    mock()
    return Response({"error": str(exc)}, status_code=500)


@asgi("/mount", exception_handlers={ValueError: value_error_handler})
async def mounted(scope: Scope, receive: Receive, send: Send) -> None:
    raise ValueError("Something went wrong")


def test_exception_handler_registered_on_handler() -> None:
    with create_test_client([mounted]) as client:
        response = client.get("/mount")
        assert response.status_code == 500
        assert response.json() == {"error": "Something went wrong"}

    mock.assert_called_once()



from litestar.plugins.base import (
    CLIPlugin,
    CLIPluginProtocol,
    DIPlugin,
    InitPlugin,
    InitPluginProtocol,
    OpenAPISchemaPlugin,
    OpenAPISchemaPluginProtocol,
    PluginProtocol,
    PluginRegistry,
    ReceiveRoutePlugin,
    SerializationPlugin,
    SerializationPluginProtocol,
)

__all__ = (
    "CLIPlugin",
    "CLIPluginProtocol",
    "DIPlugin",
    "InitPlugin",
    "InitPluginProtocol",
    "OpenAPISchemaPlugin",
    "OpenAPISchemaPluginProtocol",
    "PluginProtocol",
    "PluginRegistry",
    "ReceiveRoutePlugin",
    "SerializationPlugin",
    "SerializationPluginProtocol",
)



from __future__ import annotations

from typing import Any, Iterator
from unittest.mock import MagicMock

import pytest

from litestar import Litestar, get
from litestar._asgi.routing_trie.mapping import build_route_middleware_stack
from litestar.middleware._internal.exceptions import ExceptionHandlerMiddleware
from litestar.routes import HTTPRoute


def test_build_route_middleware_stack_no_middleware(monkeypatch: pytest.MonkeyPatch) -> None:
    # if there is no middleware for the route, then we don't need to wrap route.handle in
    # exception handling middleware. Exceptions can safely be caught by the outermost exception
    # handling middleware.

    @get("/")
    async def handler() -> None:
        pass

    app = Litestar(route_handlers=[handler], openapi_config=None)
    route = app.routes[0]
    handle_mock = MagicMock()
    monkeypatch.setattr(type(route), "handle", handle_mock)
    asgi_app = build_route_middleware_stack(app=app, route=route, route_handler=handler)
    assert asgi_app is handle_mock


def test_build_route_middleware_stack_with_middleware(monkeypatch: pytest.MonkeyPatch) -> None:
    # proves that if there is middleware, the route handler is wrapped in the exception handling
    # middleware, before being wrapped in the middleware stack.
    mock_middleware = MagicMock()
    del mock_middleware.__iter__

    @get("/", middleware=[mock_middleware])
    async def handler() -> None:
        pass

    route = HTTPRoute(path="/", route_handlers=[handler])
    build_route_middleware_stack(app=Litestar(), route=route, route_handler=handler)
    mock_middleware.assert_called_once()
    ((_, kw_args),) = mock_middleware.call_args_list
    assert isinstance(kw_args["app"], ExceptionHandlerMiddleware)


def test_build_route_middleware_stack_with_starlette_middleware(monkeypatch: pytest.MonkeyPatch) -> None:
    # test our support for starlette's Middleware class
    class Middleware:
        """A Starlette ``Middleware`` class.

        See https://github.com/encode/starlette/blob/23c81da94b57701eabd43f582093442e6811f81d/starlette/middleware/__init__.py#L4-L17
        """

        def __init__(self, cls: Any, **options: Any) -> None:
            self.cls = cls
            self.options = options

        def __iter__(self) -> Iterator[Any]:
            as_tuple = (self.cls, self.options)
            return iter(as_tuple)

    mock_middleware = MagicMock()
    mock_middleware_arg = MagicMock()
    del mock_middleware.__iter__

    @get("/", middleware=[Middleware(mock_middleware, arg=mock_middleware_arg)])  # type: ignore[list-item]
    async def handler() -> None:
        pass

    route = HTTPRoute(path="/", route_handlers=[handler])
    build_route_middleware_stack(app=Litestar(), route=route, route_handler=handler)
    ((_, kw_args),) = mock_middleware.call_args_list
    assert isinstance(kw_args["app"], ExceptionHandlerMiddleware)
    assert kw_args["arg"] is mock_middleware_arg



import sys

import pydantic
import pytest
from pydantic import v1 as pydantic_v1

from litestar import get, post
from litestar.testing import create_test_client


@pytest.mark.skipif(sys.version_info >= (3, 14), reason="not supported")
def test_app_with_v1_and_v2_models() -> None:
    class ModelV1(pydantic.v1.BaseModel):  # pyright: ignore
        foo: str

    class ModelV2(pydantic.BaseModel):
        foo: str

    @get("/v1")
    def handler_v1() -> ModelV1:
        return ModelV1(foo="bar")

    @get("/v2")
    def handler_v2() -> ModelV2:
        return ModelV2(foo="bar")

    with create_test_client([handler_v1, handler_v2]) as client:
        assert client.get("/v1").json() == {"foo": "bar"}
        assert client.get("/v2").json() == {"foo": "bar"}


@pytest.mark.skipif(sys.version_info >= (3, 14), reason="not supported")
def test_pydantic_v1_model_with_field_default() -> None:
    # https://github.com/litestar-org/litestar/issues/3471

    class TestDto(pydantic_v1.BaseModel):
        test_str: str = pydantic_v1.Field(default="some_default", max_length=100)

    @post(path="/test")
    async def test(data: TestDto) -> str:
        return "success"

    with create_test_client(route_handlers=[test]) as client:
        response = client.get("/schema/openapi.json")
        assert response.status_code == 200
        assert response.json() == {
            "components": {
                "schemas": {
                    "test_pydantic_v1_model_with_field_default.TestDto": {
                        "properties": {"test_str": {"default": "some_default", "maxLength": 100, "type": "string"}},
                        "required": [],
                        "title": "TestDto",
                        "type": "object",
                    }
                }
            },
            "info": {"title": "Litestar API", "version": "1.0.0"},
            "openapi": "3.1.0",
            "paths": {
                "/test": {
                    "post": {
                        "deprecated": False,
                        "operationId": "TestTest",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/test_pydantic_v1_model_with_field_default.TestDto"
                                    }
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "201": {
                                "content": {"text/plain": {"schema": {"type": "string"}}},
                                "description": "Document created, URL follows",
                                "headers": {},
                            },
                            "400": {
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "description": "Validation Exception",
                                            "examples": [{"detail": "Bad Request", "extra": {}, "status_code": 400}],
                                            "properties": {
                                                "detail": {"type": "string"},
                                                "extra": {
                                                    "additionalProperties": {},
                                                    "type": ["null", "object", "array"],
                                                },
                                                "status_code": {"type": "integer"},
                                            },
                                            "required": ["detail", "status_code"],
                                            "type": "object",
                                        }
                                    }
                                },
                                "description": "Bad request syntax or unsupported method",
                            },
                        },
                        "summary": "Test",
                    }
                }
            },
            "servers": [{"url": "/"}],
        }



from docs.examples.encoding_decoding.custom_type_encoding_decoding import app

from litestar.status_codes import HTTP_201_CREATED
from litestar.testing import TestClient


def test_custom_type_encoding_decoding_works() -> None:
    with TestClient(app) as client:
        response = client.post(
            "/asset",
            json={
                "user": "TenantA_Somebody",
                "name": "Some Asset",
            },
        )

        assert response.status_code == HTTP_201_CREATED



from typing import Any, Dict

from typing_extensions import Annotated

from litestar import Litestar, post
from litestar.enums import RequestEncodingType
from litestar.params import Body


@post(path="/", sync_to_thread=False)
def msgpack_handler(
    data: Annotated[Dict[str, Any], Body(media_type=RequestEncodingType.MESSAGEPACK)],
) -> Dict[str, Any]:
    # This will try to parse the request body as `MessagePack` regardless of the
    # `Content-Type`
    return data


app = Litestar(route_handlers=[msgpack_handler])



import dataclasses
import secrets
from typing import Any, Dict

from litestar import Litestar, Request, get
from litestar.connection import ASGIConnection
from litestar.security.jwt import JWTAuth, Token


@dataclasses.dataclass
class User:
    id: str


async def retrieve_user_handler(token: Token, connection: ASGIConnection) -> User:
    return User(id=token.sub)


jwt_auth = JWTAuth[User](
    token_secret=secrets.token_hex(),
    retrieve_user_handler=retrieve_user_handler,
    accepted_audiences=["https://api.testserver.local"],
    accepted_issuers=["https://auth.testserver.local"],
)


@get("/")
def handler(request: Request[User, Token, Any]) -> Dict[str, Any]:
    return {"id": request.user.id}


app = Litestar([handler], middleware=[jwt_auth.middleware])



from litestar.contrib.minijinja import MiniJinjaTemplateEngine
from litestar.template.config import TemplateConfig

template_config = TemplateConfig(engine=MiniJinjaTemplateEngine, directory="templates")



from pydantic import BaseModel, BeforeValidator, ConfigDict, PlainSerializer, WithJsonSchema
from typing_extensions import Annotated

from litestar import Litestar, post


class TenantUser:
    """Custom Type that represents a user associated to a tenant

    Parsed from / serialized to a combined tenant + user id string of the form

        TENANTPREFIX_USERID

    i.e. separated by underscore.
    """

    tenant_prefix: str
    user_id: str

    def __init__(self, tenant_prefix: str, user_id: str) -> None:
        self.tenant_prefix = tenant_prefix
        self.user_id = user_id

    @classmethod
    def from_string(cls, s: str) -> "TenantUser":
        splits = s.split("_", maxsplit=1)
        if len(splits) < 2:
            raise ValueError(
                "Could not split up tenant user id string. "
                "Expecting underscore for separation of tenant prefix and user id."
            )
        return cls(tenant_prefix=splits[0], user_id=splits[1])

    def to_combined_str(self) -> str:
        return self.tenant_prefix + "_" + self.user_id


PydAnnotatedTenantUser = Annotated[
    TenantUser,
    BeforeValidator(TenantUser.from_string),
    PlainSerializer(lambda x: x.to_combined_str(), return_type=str),
    WithJsonSchema({"type": "string"}, mode="serialization"),
]


class UserAsset(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    user: PydAnnotatedTenantUser
    name: str


@post("/asset", sync_to_thread=False)
def create_asset(
    data: UserAsset,
) -> UserAsset:
    assert isinstance(data.user, TenantUser)
    return data


app = Litestar(
    [create_asset],
)

# run: /asset -X POST -H "Content-Type: application/json" -d '{"name":"SomeAsset","user":"TenantA_Somebody"}'



from typing import Any, Dict

from litestar import Litestar, Request, get
from litestar.middleware.session.server_side import ServerSideSessionConfig
from litestar.testing import TestClient

session_config = ServerSideSessionConfig()


@get(path="/test", sync_to_thread=False)
def get_session_data(request: Request) -> Dict[str, Any]:
    return request.session


app = Litestar(route_handlers=[get_session_data], middleware=[session_config.middleware], debug=True)


def test_get_session_data() -> None:
    with TestClient(app=app, session_config=session_config) as client:
        client.set_session_data({"foo": "bar"})
        assert client.get("/test").json() == {"foo": "bar"}



from dataclasses import dataclass

from litestar import post
from litestar.datastructures.secret_values import SecretString


@dataclass
class Sensitive:
    value: SecretString


@post(sync_to_thread=False)
def post_handler(data: Sensitive) -> Sensitive:
    return data



import anyio

from litestar import Litestar, get
from litestar.enums import ScopeType
from litestar.exceptions import ClientException
from litestar.middleware import ASGIMiddleware
from litestar.types import ASGIApp, Receive, Scope, Send


class TimeoutMiddleware(ASGIMiddleware):
    # we can configure some things on the class level here, related to when our
    # middleware should be applied.

    # if the requests' 'scope["type"]' is not "http", the middleware will be skipped
    scopes = (ScopeType.HTTP,)

    # if the handler for a request has set an opt of 'no_timeout=True', the middleware
    # will be skipped
    exclude_opt_key = "no_timeout"

    # the base class does not define an '__init__' method, so we're free to overwrite
    # this, which we're making use of to add some configuration
    def __init__(
        self,
        timeout: float,
        exclude_path_pattern: str | tuple[str, ...] | None = None,
    ) -> None:
        self.timeout = timeout

        # we can also dynamically configure the options provided by the base class on
        # the instance level
        self.exclude_path_pattern = exclude_path_pattern

    async def handle(self, scope: Scope, receive: Receive, send: Send, next_app: ASGIApp) -> None:
        try:
            with anyio.fail_after(self.timeout):
                # call the next app in the chain
                await next_app(scope, receive, send)
        except TimeoutError:
            # if the request has timed out, raise an exception. since the whole
            # application is wrapped in an exception handling middleware, it will
            # transform this exception into a response for us
            raise ClientException(status_code=408) from None


@get("/", no_timeout=True)
async def handler_with_opt_skip() -> None:
    pass


@get("/not-this-path")
async def handler_with_path_skip() -> None:
    pass


app = Litestar(
    route_handlers=[
        handler_with_opt_skip,
        handler_with_path_skip,
    ],
    middleware=[TimeoutMiddleware(timeout=5)],
)



from typing import Literal, Union

from litestar import Litestar, post


@post("/")
async def query_type_test(param: Union[Literal["1"], None]) -> None:
    return None


app = Litestar(route_handlers=[query_type_test])



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Generic, TypeVar, Union

__all__ = (
    "SecretBytes",
    "SecretString",
    "SecretT",
    "SecretValue",
)

# NOTE: Union instead of ` | ` is used for compatibility with Sphinx autodoc.
#  - Sphinx autodoc doesn't handle string annotations for "bound" parameter in TypeVar.
#  - We cannot use `|` without string annotations due to supporting < python 3.10.
SecretT = TypeVar("SecretT", bound=Union[str, bytes])
"""Type that represents a secret value of type ``str`` or ``bytes``."""


class SecretValue(ABC, Generic[SecretT]):
    """Represents a secret value that can be of type `str` or `bytes`."""

    def __init__(self, secret_value: SecretT) -> None:
        """Initializes a :class:`SecretValue` object with a secret value of type ``str`` or ``bytes``.

        Args:
            secret_value (str | bytes): The secret value to be encapsulated.
        """

        self._secret_value = secret_value

    def get_secret(self) -> SecretT:
        """Returns the actual secret value.

        Returns:
            str | bytes: The secret value.
        """

        return self._secret_value

    @abstractmethod
    def get_obscured(self) -> SecretT:
        """Return the hidden representation of the secret value.

        Raises:
            NotImplementedError: Always raised to enforce implementation in subclasses.
        """

        raise NotImplementedError("Subclasses must implement get_obscured")

    def __str__(self) -> str:
        """Returns a string representation of the hidden secret value.

        Returns:
            str: String representation of the hidden secret value.
        """

        return str(self.get_obscured())

    def __repr__(self) -> str:
        """Returns a string representation of the object for debugging purposes.

        Returns:
            str: String representation of the object.
        """

        class_name = self.__class__.__name__
        return f"{class_name}({self.get_obscured()!r})"


class SecretString(SecretValue[str]):
    """Represents a secret string value."""

    def get_obscured(self) -> str:
        """Overrides the base class method to return the hidden string value.

        Returns:
            str: The hidden string representation of the secret value.
        """

        return "******"


class SecretBytes(SecretValue[bytes]):
    """Represents a secret bytes value."""

    def get_obscured(self) -> bytes:
        """Overrides the base class method to return the hidden bytes value.

        Returns:
            bytes: The hidden bytes representation of the secret value.
        """

        return b"******"



from fsspec.implementations.ftp import FTPFileSystem

from litestar import Litestar
from litestar.static_files import create_static_files_router

app = Litestar(
    route_handlers=[
        create_static_files_router(
            path="/static",
            directories=["assets"],
            file_system=FTPFileSystem(host="127.0.0.1"),
        ),
    ]
)



from __future__ import annotations

import abc
import warnings
from abc import abstractmethod
from typing import TYPE_CHECKING, Any, Callable, Protocol, runtime_checkable

from litestar.enums import ScopeType
from litestar.middleware._utils import (
    build_exclude_path_pattern,
    should_bypass_for_path_pattern,
    should_bypass_middleware,
)
from litestar.utils.deprecation import warn_deprecation

__all__ = (
    "ASGIMiddleware",
    "AbstractMiddleware",
    "DefineMiddleware",
    "MiddlewareProtocol",
)


if TYPE_CHECKING:
    from litestar.types import Scopes
    from litestar.types.asgi_types import ASGIApp, Receive, Scope, Send


@runtime_checkable
class MiddlewareProtocol(Protocol):
    """Abstract middleware protocol."""

    __slots__ = ("app",)

    app: ASGIApp

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        """Execute the ASGI middleware.

        Called by the previous middleware in the stack if a response is not awaited prior.

        Upon completion, middleware should call the next ASGI handler and await it - or await a response created in its
        closure.

        Args:
            scope: The ASGI connection scope.
            receive: The ASGI receive function.
            send: The ASGI send function.

        Returns:
            None
        """


class DefineMiddleware:
    """Container enabling passing ``*args`` and ``**kwargs`` to Middleware class constructors and factory functions."""

    __slots__ = ("args", "kwargs", "middleware")

    def __init__(self, middleware: Callable[..., ASGIApp], *args: Any, **kwargs: Any) -> None:
        """Initialize ``DefineMiddleware``.

        Args:
            middleware: A callable that returns an ASGIApp.
            *args: Positional arguments to pass to the callable.
            **kwargs: Key word arguments to pass to the callable.

        Notes:
            The callable will be passed a kwarg ``app``, which is the next ASGI app to call in the middleware stack.
            It therefore must define such a kwarg.
        """
        self.middleware = middleware
        self.args = args
        self.kwargs = kwargs

    def __call__(self, app: ASGIApp) -> ASGIApp:
        """Call the middleware constructor or factory.

        Args:
            app: An ASGIApp, this value is the next ASGI handler to call in the middleware stack.

        Returns:
            Calls :class:`DefineMiddleware.middleware <.DefineMiddleware>` and returns the ASGIApp created.
        """

        return self.middleware(*self.args, app=app, **self.kwargs)


class AbstractMiddleware:
    """Abstract middleware providing base functionality common to all middlewares, for dynamically engaging/bypassing
    the middleware based on paths, ``opt``-keys and scope types.

    When implementing new middleware, this class should be used as a base.
    """

    scopes: Scopes = {ScopeType.HTTP, ScopeType.WEBSOCKET}
    exclude: str | list[str] | None = None
    exclude_opt_key: str | None = None

    def __init__(
        self,
        app: ASGIApp,
        exclude: str | list[str] | None = None,
        exclude_opt_key: str | None = None,
        scopes: Scopes | None = None,
    ) -> None:
        """Initialize the middleware.

        Args:
            app: The ``next`` ASGI app to call.
            exclude: A pattern or list of patterns to match against a request's path.
                If a match is found, the middleware will be skipped.
            exclude_opt_key: An identifier that is set in the route handler
                ``opt`` key which allows skipping the middleware.
            scopes: ASGI scope types, should be a set including
                either or both 'ScopeType.HTTP' and 'ScopeType.WEBSOCKET'.
        """
        self.app = app
        self.scopes = scopes or self.scopes
        self.exclude_opt_key = exclude_opt_key or self.exclude_opt_key
        self.exclude_pattern = build_exclude_path_pattern(exclude=(exclude or self.exclude), middleware_cls=type(self))

    @classmethod
    def __init_subclass__(cls, **kwargs: Any) -> None:
        if not any(c.__module__.startswith("litestar") and c is not AbstractMiddleware for c in cls.mro()):
            # we don't want to warn about usage of 'AbstractMiddleware' if users aren't
            # directly subclassing it, i.e. they're subclassing another Litestar
            # middleware which itself subclasses 'AbstractMiddleware'
            warn_deprecation(
                version="2.15",
                deprecated_name="AbstractMiddleware",
                kind="class",
                alternative="litestar.middleware.ASGIMiddleware",
            )

        super().__init_subclass__(**kwargs)

        original__call__ = cls.__call__

        async def wrapped_call(self: AbstractMiddleware, scope: Scope, receive: Receive, send: Send) -> None:
            if should_bypass_middleware(
                scope=scope,
                scopes=self.scopes,
                exclude_path_pattern=self.exclude_pattern,
                exclude_opt_key=self.exclude_opt_key,
            ):
                await self.app(scope, receive, send)
            else:
                await original__call__(self, scope, receive, send)  # pyright: ignore

        # https://github.com/python/mypy/issues/2427#issuecomment-384229898
        setattr(cls, "__call__", wrapped_call)

    @abstractmethod
    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        """Execute the ASGI middleware.

        Called by the previous middleware in the stack if a response is not awaited prior.

        Upon completion, middleware should call the next ASGI handler and await it - or await a response created in its
        closure.

        Args:
            scope: The ASGI connection scope.
            receive: The ASGI receive function.
            send: The ASGI send function.

        Returns:
            None
        """
        raise NotImplementedError("abstract method must be implemented")


class ASGIMiddleware(abc.ABC):
    """An abstract base class to easily construct ASGI middlewares, providing functionality
    to dynamically skip the middleware based on ASGI ``scope["type"]``, handler ``opt``
    keys or path patterns and a simple way to pass configuration to middlewares.

    This base class does not implement an ``__init__`` method, so subclasses are free
    to use it to customize the middleware's configuration.

    .. important::
        An instance of the individual middlewares will be created *once* and used to
        build up the internal middleware stack. As such, middlewares should *not* be
        stateful, as this state will be shared across all requests.
        Any connection-specific state should be scoped to the `handle` implementation.
        Not doing so would typically lead to conflicting variable reads / writes across
        requests, and - most likely - bugs.


    .. code-block:: python

        class MyMiddleware(ASGIMiddleware):
            scopes = (ScopeType.HTTP,)
            exclude_path_pattern = ("/not/this/path",)
            exclude_opt_key = "exclude_my_middleware"

            def __init__(self, my_logger: Logger) -> None:
                self.logger = my_logger

            async def handle(
                self, scope: Scope, receive: Receive, send: Send, next_app: ASGIApp
            ) -> None:
                self.logger.debug("Received request for path %s", scope["path"])
                await next_app(scope, receive, send)
                self.logger.debug("Processed request for path %s", scope["path"])


        app = Litestar(..., middleware=[MyMiddleware(logger=my_logger)])

    .. versionadded:: 2.15
    """

    scopes: tuple[ScopeType, ...] = (
        ScopeType.HTTP,
        ScopeType.WEBSOCKET,
        ScopeType.ASGI,
    )
    exclude_path_pattern: str | tuple[str, ...] | None = None
    exclude_opt_key: str | None = None

    should_bypass_for_scope: Callable[[Scope], bool] | None = None
    r"""
    A callable that takes in the :class:`~litestar.types.Scope` of the current
    connection and returns a boolean, indicating if the middleware should be skipped for
    the current request.

    This can for example be used to exclude a middleware based on a dynamic path::

        should_bypass_for_scope = lambda scope: scope["path"].endswith(".jpg")

    Applied to a route with a dynamic path like ``/static/{file_name:str}``, it would
    be skipped *only* if ``file_name`` has a ``.jpg`` extension.

    .. versionadded:: 2.19
    """

    def __call__(self, app: ASGIApp) -> ASGIApp:
        """Create the actual middleware callable"""
        handle = self.handle
        exclude_pattern = build_exclude_path_pattern(exclude=self.exclude_path_pattern, middleware_cls=type(self))
        scopes = set(self.scopes)
        exclude_opt_key = self.exclude_opt_key
        should_bypass_for_scope = self.should_bypass_for_scope

        def exclude_pattern_matches_handler_path(scope: Scope) -> bool:
            if exclude_pattern is None:
                return False
            handler = scope["route_handler"]
            return any(exclude_pattern.search(path) for path in handler.paths)

        async def middleware(scope: Scope, receive: Receive, send: Send) -> None:
            path_excluded = False
            if (
                should_bypass_middleware(
                    scope=scope,
                    scopes=scopes,  # type: ignore[arg-type]
                    exclude_opt_key=exclude_opt_key,
                )
                or (path_excluded := should_bypass_for_path_pattern(scope, exclude_pattern))
                or (should_bypass_for_scope and should_bypass_for_scope(scope))
            ):
                if path_excluded and exclude_pattern is not None and not exclude_pattern_matches_handler_path(scope):
                    warnings.warn(
                        f"{type(self).__name__}.exclude_path_pattern={exclude_pattern.pattern!r} "
                        "did match the request path but did not match the route "
                        "handler's path. When upgrading to Litestar 3, this middleware "
                        "would NOT be excluded. To keep the current behaviour, use "
                        "'should_bypass_for_scope' instead.",
                        category=DeprecationWarning,
                        stacklevel=2,
                    )

                await app(scope, receive, send)
            else:
                if exclude_pattern is not None and exclude_pattern_matches_handler_path(scope):
                    warnings.warn(
                        f"{type(self).__name__}.exclude_path_pattern={exclude_pattern.pattern!r} "
                        "did not match the request path but did match the route "
                        "handler's path. When upgrading to Litestar 3, this middleware "
                        "would be excluded. To keep the current behaviour, use "
                        "'should_bypass_for_scope' instead.",
                        category=DeprecationWarning,
                        stacklevel=2,
                    )
                await handle(scope=scope, receive=receive, send=send, next_app=app)

        return middleware

    @abc.abstractmethod
    async def handle(self, scope: Scope, receive: Receive, send: Send, next_app: ASGIApp) -> None:
        """Handle ASGI call.

        Args:
            scope: The ASGI connection scope.
            receive: The ASGI receive function.
            send: The ASGI send function
            next_app: The next ASGI application in the middleware stack to call
        """

        raise NotImplementedError



from __future__ import annotations

import dataclasses
import functools
import warnings
from typing import TYPE_CHECKING, Any, AsyncGenerator, Awaitable, Callable, Mapping, cast

import anyio
from msgspec.json import Encoder as JsonEncoder
from typing_extensions import Self

from litestar.exceptions import ImproperlyConfiguredException, LitestarWarning, WebSocketDisconnect
from litestar.handlers.websocket_handlers.route_handler import WebsocketRouteHandler
from litestar.types import Empty
from litestar.types.builtin_types import NoneType
from litestar.typing import FieldDefinition
from litestar.utils.signature import ParsedSignature

if TYPE_CHECKING:
    from litestar import Litestar, WebSocket
    from litestar.dto import AbstractDTO
    from litestar.types import Dependencies, EmptyType, ExceptionHandler, Guard, Middleware, TypeEncodersMap
    from litestar.types.asgi_types import WebSocketMode


async def send_websocket_stream(
    socket: WebSocket,
    stream: AsyncGenerator[Any, Any],
    *,
    close: bool = True,
    mode: WebSocketMode = "text",
    send_handler: Callable[[WebSocket, Any], Awaitable[Any]] | None = None,
    listen_for_disconnect: bool = False,
    warn_on_data_discard: bool = True,
) -> None:
    """Stream data to the ``socket`` from an asynchronous generator.

    Example:
        Sending the current time to the connected client every 0.5 seconds:

        .. code-block:: python

            async def stream_current_time() -> AsyncGenerator[str, None]:
                while True:
                    yield str(time.time())
                    await asyncio.sleep(0.5)


            @websocket("/time")
            async def time_handler(socket: WebSocket) -> None:
                await socket.accept()
                await send_websocket_stream(
                    socket,
                    stream_current_time(),
                    listen_for_disconnect=True,
                )


    Args:
        socket: The :class:`~litestar.connection.WebSocket` to send to
        stream: An asynchronous generator yielding data to send
        close: If ``True``, close the socket after the generator is exhausted
        mode: WebSocket mode to use for sending when no ``send_handler`` is specified
        send_handler: Callable to handle the send process. If ``None``, defaults to ``type(socket).send_data``
        listen_for_disconnect: If ``True``, listen for client disconnects in the background. If a client disconnects,
            stop the generator and cancel sending data. Should always be ``True`` unless disconnects are handled
            elsewhere, for example by reading data from the socket concurrently. Should never be set to ``True`` when
            reading data from socket concurrently, as it can lead to data loss
        warn_on_data_discard: If ``True`` and ``listen_for_disconnect=True``, warn if during listening for client
            disconnects, data is received from the socket
    """
    if send_handler is None:
        send_handler = functools.partial(type(socket).send_data, mode=mode)

    async def send_stream() -> None:
        try:
            # client might have disconnected elsewhere, so we stop sending
            while socket.connection_state != "disconnect":
                await send_handler(socket, await stream.__anext__())
        except StopAsyncIteration:
            pass

    if listen_for_disconnect:
        # wrap 'send_stream' and disconnect listener, so they'll cancel the other once
        # one of the finishes
        async def wrapped_stream() -> None:
            await send_stream()
            # stream exhausted, we can stop listening for a disconnect
            tg.cancel_scope.cancel()

        async def disconnect_listener() -> None:
            try:
                # run this in a loop - we might receive other data than disconnects.
                # listen_for_disconnect is explicitly not safe when consuming WS data
                # in other places, so discarding that data here is fine
                while True:
                    await socket.receive_data("text")
                    if warn_on_data_discard:
                        warnings.warn(
                            "received data from websocket while listening for client "
                            "disconnect in a websocket_stream. listen_for_disconnect "
                            "is not safe to use when attempting to receive data from "
                            "the same socket concurrently with a websocket_stream. set "
                            "listen_for_disconnect=False if you're attempting to "
                            "receive data from this socket or set "
                            "warn_on_data_discard=False to disable this warning",
                            stacklevel=2,
                            category=LitestarWarning,
                        )

            except WebSocketDisconnect:
                # client disconnected, we can stop streaming
                tg.cancel_scope.cancel()

        async with anyio.create_task_group() as tg:
            tg.start_soon(wrapped_stream)
            tg.start_soon(disconnect_listener)

    else:
        await send_stream()

    if close and socket.connection_state != "disconnect":
        await socket.close()


def websocket_stream(
    path: str | list[str] | None = None,
    *,
    dependencies: Dependencies | None = None,
    exception_handlers: dict[int | type[Exception], ExceptionHandler] | None = None,
    guards: list[Guard] | None = None,
    middleware: list[Middleware] | None = None,
    name: str | None = None,
    opt: dict[str, Any] | None = None,
    signature_namespace: Mapping[str, Any] | None = None,
    websocket_class: type[WebSocket] | None = None,
    mode: WebSocketMode = "text",
    return_dto: type[AbstractDTO] | None | EmptyType = Empty,
    type_encoders: TypeEncodersMap | None = None,
    listen_for_disconnect: bool = True,
    warn_on_data_discard: bool = True,
    **kwargs: Any,
) -> Callable[[Callable[..., AsyncGenerator[Any, Any]]], WebsocketRouteHandler]:
    """Create a WebSocket handler that accepts a connection and sends data to it from an
    async generator.

    Example:
        Sending the current time to the connected client every 0.5 seconds:

        .. code-block:: python

            @websocket_stream("/time")
            async def send_time() -> AsyncGenerator[str, None]:
                while True:
                    yield str(time.time())
                    await asyncio.sleep(0.5)

    Args:
        path: A path fragment for the route handler function or a sequence of path fragments. If not given defaults
            to ``/``
        dependencies: A string keyed mapping of dependency :class:`Provider <.di.Provide>` instances.
        exception_handlers: A mapping of status codes and/or exception types to handler functions.
        guards: A sequence of :class:`Guard <.types.Guard>` callables.
        middleware: A sequence of :class:`Middleware <.types.Middleware>`.
        name: A string identifying the route handler.
        opt: A string keyed mapping of arbitrary values that can be accessed in :class:`Guards <.types.Guard>` or
            wherever you have access to :class:`Request <.connection.Request>` or
            :class:`ASGI Scope <.types.Scope>`.
        signature_namespace: A mapping of names to types for use in forward reference resolution during signature modelling.
        websocket_class: A custom subclass of :class:`WebSocket <.connection.WebSocket>` to be used as route handler's
            default websocket class.
        mode: WebSocket mode used for sending
        return_dto: :class:`AbstractDTO <.dto.base_dto.AbstractDTO>` to use for serializing outbound response data.
        type_encoders: A mapping of types to callables that transform them into types supported for serialization.
        listen_for_disconnect: If ``True``, listen for client disconnects in the background. If a client disconnects,
            stop the generator and cancel sending data. Should always be ``True`` unless disconnects are handled
            elsewhere, for example by reading data from the socket concurrently. Should never be set to ``True`` when
            reading data from socket concurrently, as it can lead to data loss
        warn_on_data_discard: If ``True`` and ``listen_for_disconnect=True``, warn if during listening for client
            disconnects, data is received from the socket
        **kwargs: Any additional kwarg - will be set in the opt dictionary.
    """

    def decorator(fn: Callable[..., AsyncGenerator[Any, Any]]) -> WebsocketRouteHandler:
        return WebSocketStreamHandler(
            path=path,
            dependencies=dependencies,
            exception_handlers=exception_handlers,
            guards=guards,
            middleware=middleware,
            name=name,
            opt=opt,
            signature_namespace=signature_namespace,
            websocket_class=websocket_class,
            return_dto=return_dto,
            type_encoders=type_encoders,
            **kwargs,
        )(
            _WebSocketStreamOptions(
                generator_fn=fn,
                send_mode=mode,
                listen_for_disconnect=listen_for_disconnect,
                warn_on_data_discard=warn_on_data_discard,
            )
        )

    return decorator


class WebSocketStreamHandler(WebsocketRouteHandler):
    __slots__ = ("_ws_stream_options",)
    _ws_stream_options: _WebSocketStreamOptions

    def __call__(self, fn: _WebSocketStreamOptions) -> Self:  # type: ignore[override]
        self._ws_stream_options = fn
        self._fn = self._ws_stream_options.generator_fn  # type: ignore[assignment]
        return self

    def on_registration(self, app: Litestar) -> None:
        parsed_handler_signature = parsed_stream_fn_signature = ParsedSignature.from_fn(
            self.fn, self.resolve_signature_namespace()
        )

        if not parsed_stream_fn_signature.return_type.is_subclass_of(AsyncGenerator):
            raise ImproperlyConfiguredException(
                f"Route handler {self}: 'websocket_stream' handlers must return an "
                f"'AsyncGenerator', not {type(parsed_stream_fn_signature.return_type.raw)!r}"
            )

        # important not to use 'self._ws_stream_options.generator_fn' here; This would
        # break in cases the decorator has been used inside a controller, as it would
        # be a reference to the unbound method. The bound method is patched in later
        # after the controller has been initialized. This is a workaround that should
        # go away with v3.0's static handlers
        stream_fn = cast("Callable[..., AsyncGenerator[Any, Any]]", self.fn)

        # construct a fake signature for the kwargs modelling, using the generator
        # function passed to the handler as a base, to include all the dependencies,
        # params, injection kwargs, etc. + 'socket', so DI works properly, but the
        # signature looks to kwargs/signature modelling like a plain '@websocket'
        # handler that returns 'None'
        parsed_handler_signature = dataclasses.replace(
            parsed_handler_signature, return_type=FieldDefinition.from_annotation(NoneType)
        )
        receives_socket_parameter = "socket" in parsed_stream_fn_signature.parameters

        if not receives_socket_parameter:
            parsed_handler_signature = dataclasses.replace(
                parsed_handler_signature,
                parameters={
                    **parsed_handler_signature.parameters,
                    "socket": FieldDefinition.from_annotation("WebSocket", name="socket"),
                },
            )

        self._parsed_fn_signature = parsed_handler_signature
        self._parsed_return_field = parsed_stream_fn_signature.return_type.inner_types[0]

        json_encoder = JsonEncoder(enc_hook=self.default_serializer)
        return_dto = self.resolve_return_dto()

        # make sure the closure doesn't capture self._ws_stream / self
        send_mode: WebSocketMode = self._ws_stream_options.send_mode  # pyright: ignore
        listen_for_disconnect = self._ws_stream_options.listen_for_disconnect
        warn_on_data_discard = self._ws_stream_options.warn_on_data_discard

        async def send_handler(socket: WebSocket, data: Any) -> None:
            if isinstance(data, (str, bytes)):
                await socket.send_data(data=data, mode=send_mode)
                return

            if return_dto:
                encoded_data = return_dto(socket).data_to_encodable_type(data)
                data = json_encoder.encode(encoded_data)
                await socket.send_data(data=data, mode=send_mode)
                return

            data = json_encoder.encode(data)
            await socket.send_data(data=data, mode=send_mode)

        @functools.wraps(stream_fn)
        async def handler_fn(*args: Any, socket: WebSocket, **kw: Any) -> None:
            if receives_socket_parameter:
                kw["socket"] = socket

            await send_websocket_stream(
                socket=socket,
                stream=stream_fn(*args, **kw),
                mode=send_mode,
                close=True,
                listen_for_disconnect=listen_for_disconnect,
                warn_on_data_discard=warn_on_data_discard,
                send_handler=send_handler,
            )

        self._fn = handler_fn

        super().on_registration(app)


class _WebSocketStreamOptions:
    def __init__(
        self,
        generator_fn: Callable[..., AsyncGenerator[Any, Any]],
        listen_for_disconnect: bool,
        warn_on_data_discard: bool,
        send_mode: WebSocketMode,
    ) -> None:
        self.generator_fn = generator_fn
        self.listen_for_disconnect = listen_for_disconnect
        self.warn_on_data_discard = warn_on_data_discard
        self.send_mode = send_mode



from __future__ import annotations

from dataclasses import dataclass
from uuid import UUID

from litestar import Litestar, patch
from litestar.dto import DataclassDTO, DTOConfig, DTOData


@dataclass
class Person:
    id: UUID
    name: str
    age: int


class PatchDTO(DataclassDTO[Person]):
    """Don't allow client to set the id, and allow partial updates."""

    config = DTOConfig(exclude={"id"}, partial=True)


peter_uuid = UUID("f32ff2ce-e32f-4537-9dc0-26e7599f1380")
database = {peter_uuid: Person(id=peter_uuid, name="Peter", age=40)}


@patch("/person/{person_id:uuid}", dto=PatchDTO, return_dto=None, sync_to_thread=False)
def update_person(person_id: UUID, data: DTOData[Person]) -> Person:
    """Partially update a person."""
    return data.update_instance(database[person_id])


app = Litestar(route_handlers=[update_person])

# run: /person/f32ff2ce-e32f-4537-9dc0-26e7599f1380 -X PATCH -H "Content-Type: application/json" -d '{"name":"Peter Pan"}'



"""
Assemble components into an app that shall be tested
"""

from typing import AsyncGenerator

from litestar import Litestar, get
from litestar.response import ServerSentEvent
from litestar.types import SSEData


async def generator(topic: str) -> AsyncGenerator[SSEData, None]:
    count = 0
    while count < 2:
        yield topic
        count += 1


@get("/notify/{topic:str}")
async def get_notified(topic: str) -> ServerSentEvent:
    return ServerSentEvent(generator(topic), event_type="Notifier")


app = Litestar(route_handlers=[get_notified])



from litestar import Litestar, MediaType, Request, Response, get


@get("/resource", sync_to_thread=False)
def retrieve_resource(request: Request) -> Response[bytes]:
    provided_types = [MediaType.TEXT, MediaType.HTML, "application/xml"]
    preferred_type = request.accept.best_match(provided_types, default=MediaType.TEXT)

    content = None
    if preferred_type == MediaType.TEXT:
        content = b"Hello World!"
    elif preferred_type == MediaType.HTML:
        content = b"<h1>Hello World!</h1>"
    elif preferred_type == "application/xml":
        content = b"<xml><msg>Hello World!</msg></xml>"
    return Response(content=content, media_type=preferred_type)


app = Litestar(route_handlers=[retrieve_resource])

# run: /resource
# run: /resource -H "Accept: text/html"
# run: /resource -H "Accept: application/xml"



"""Plugin for creating and retrieving flash messages."""

from __future__ import annotations

from contextlib import suppress
from dataclasses import dataclass
from typing import TYPE_CHECKING, Any, Mapping

import litestar.exceptions
from litestar.exceptions import MissingDependencyException
from litestar.middleware import DefineMiddleware
from litestar.middleware.session import SessionMiddleware
from litestar.plugins import InitPlugin
from litestar.security.session_auth.middleware import MiddlewareWrapper
from litestar.template.base import _get_request_from_context
from litestar.utils.predicates import is_class_and_subclass

if TYPE_CHECKING:
    from collections.abc import Callable

    from litestar.config.app import AppConfig
    from litestar.connection.base import ASGIConnection
    from litestar.template import TemplateConfig


@dataclass
class FlashConfig:
    """Configuration for Flash messages."""

    template_config: TemplateConfig


class FlashPlugin(InitPlugin):
    """Flash messages Plugin."""

    def __init__(self, config: FlashConfig):
        """Initialize the plugin.

        Args:
            config: Configuration for flash messages, including the template engine instance.
        """
        self.config = config

    def on_app_init(self, app_config: AppConfig) -> AppConfig:
        """Register the message callable on the template engine instance.

        Args:
            app_config: The application configuration.

        Returns:
            The application configuration with the message callable registered.
        """
        for mw in app_config.middleware:
            if isinstance(mw, DefineMiddleware) and is_class_and_subclass(
                mw.middleware, (MiddlewareWrapper, SessionMiddleware)
            ):
                break
        else:
            raise litestar.exceptions.ImproperlyConfiguredException("Flash messages require a session middleware.")
        template_callable: Callable[[Any], Any] = get_flashes
        with suppress(MissingDependencyException):
            from litestar.contrib.minijinja import MiniJinjaTemplateEngine, _transform_state

            if isinstance(self.config.template_config.engine_instance, MiniJinjaTemplateEngine):
                template_callable = _transform_state(get_flashes)

        self.config.template_config.engine_instance.register_template_callable("get_flashes", template_callable)  # pyright: ignore[reportGeneralTypeIssues]
        return app_config


def flash(
    request: ASGIConnection[Any, Any, Any, Any],
    message: Any,
    category: str,
) -> None:
    request.session.setdefault("_messages", []).append({"message": message, "category": category})


def get_flashes(context: Mapping[str, Any]) -> Any:
    return _get_request_from_context(context).session.pop("_messages", [])



import asyncio
from typing import AsyncGenerator

from app.lib import ping_external_resource

from litestar import Litestar, websocket_stream

RESOURCE_LOCK = asyncio.Lock()


async def acquire_lock() -> AsyncGenerator[None, None]:
    async with RESOURCE_LOCK:
        yield


@websocket_stream("/")
async def ping(lock: asyncio.Lock) -> AsyncGenerator[float, None]:
    while True:
        alive = await ping_external_resource()
        yield alive
        await asyncio.sleep(1)


app = Litestar([ping], dependencies={"lock": acquire_lock})



from unittest.mock import ANY

from litestar.status_codes import HTTP_201_CREATED
from litestar.testing.client import TestClient


def test_create_user(user_data: dict) -> None:
    from docs.examples.data_transfer_objects.factory.dto_data_problem_statement import app

    with TestClient(app=app) as client:
        response = client.post("/users", json=user_data)

    assert response.status_code == HTTP_201_CREATED
    assert response.json() == {"id": ANY, "name": "Mr Sunglass", "email": "mr.sunglass@example.com", "age": 30}



from typing import Dict

from typing_extensions import Annotated

from litestar import Litestar, get
from litestar.params import Parameter


@get("/", sync_to_thread=False)
def index(snake_case: Annotated[str, Parameter(query="camelCase")]) -> Dict[str, str]:
    return {"param": snake_case}


app = Litestar(route_handlers=[index])

# run: /?camelCase=foo



from typing import Dict, Generator

from litestar import Litestar, get
from litestar.di import Provide

CONNECTION = {"open": False}


def generator_function() -> Generator[Dict[str, bool], None, None]:
    """Set connection to open and close it after the handler returns."""
    CONNECTION["open"] = True
    yield CONNECTION
    CONNECTION["open"] = False


@get("/", dependencies={"conn": Provide(generator_function)})
def index(conn: Dict[str, bool]) -> Dict[str, bool]:
    """Return the current connection state."""
    return conn


app = Litestar(route_handlers=[index])



from typing import Dict

from litestar import Litestar, get
from litestar.logging.config import LoggingConfig
from litestar.middleware.logging import LoggingMiddlewareConfig

logging_middleware_config = LoggingMiddlewareConfig()


@get("/", sync_to_thread=False)
def my_handler() -> Dict[str, str]:
    return {"hello": "world"}


app = Litestar(
    route_handlers=[my_handler],
    logging_config=LoggingConfig(),
    middleware=[logging_middleware_config.middleware],
)



from litestar import Litestar
from litestar.channels import ChannelsPlugin
from litestar.channels.backends.memory import MemoryChannelsBackend

channels_plugin = ChannelsPlugin(
    backend=MemoryChannelsBackend(),
    channels=["foo", "bar"],
    create_ws_route_handlers=True,
)

app = Litestar(plugins=[channels_plugin])



from starlette.applications import Starlette
from starlette.requests import Request
from starlette.responses import PlainTextResponse
from starlette.routing import Route

from litestar import Litestar, asgi, get


@get("/")
async def litestar_index() -> str:
    return "Hello, world!"


async def starlette_index(_: Request) -> PlainTextResponse:
    return PlainTextResponse({"app": "starlette"})


foo_app = asgi(path="/foo", is_mount=True)(Starlette(routes=[Route("/", starlette_index)]))
bar_app = asgi(path="/bar", is_mount=True)(Litestar(route_handlers=[litestar_index]))

app = Litestar(route_handlers=[foo_app, bar_app])

if __name__ == "__main__":
    import uvicorn

    uvicorn.run(app)



from typing import Any, AsyncGenerator, Callable, Dict, Generator
from unittest.mock import MagicMock

import pytest
from pytest import FixtureRequest

from litestar import Response, WebSocket, get, websocket
from litestar.response.base import ASGIResponse
from litestar.testing import create_test_client


@pytest.fixture
def cleanup_mock() -> MagicMock:
    return MagicMock()


@pytest.fixture
def exception_mock() -> MagicMock:
    return MagicMock()


@pytest.fixture
def finally_mock() -> MagicMock:
    return MagicMock()


@pytest.fixture
def generator_dependency(
    cleanup_mock: MagicMock, exception_mock: MagicMock, finally_mock: MagicMock
) -> Callable[[], Generator[str, None, None]]:
    def dependency() -> Generator[str, None, None]:
        try:
            yield "hello"
            cleanup_mock()
        except ValueError:
            exception_mock()
        finally:
            finally_mock()

    return dependency


@pytest.fixture
def async_generator_dependency(
    cleanup_mock: MagicMock, exception_mock: MagicMock, finally_mock: MagicMock
) -> Callable[[], AsyncGenerator[str, None]]:
    async def dependency() -> AsyncGenerator[str, None]:
        try:
            yield "hello"
            cleanup_mock()
        except ValueError:
            exception_mock()
        finally:
            finally_mock()

    return dependency


@pytest.mark.parametrize("cache", [False, True])
@pytest.mark.parametrize("dependency_fixture", ["generator_dependency", "async_generator_dependency"])
def test_generator_dependency(
    cache: bool,
    request: FixtureRequest,
    dependency_fixture: str,
    cleanup_mock: MagicMock,
    exception_mock: MagicMock,
    finally_mock: MagicMock,
) -> None:
    dependency = request.getfixturevalue(dependency_fixture)

    @get("/", dependencies={"dep": dependency}, cache=cache)
    def handler(dep: str) -> Dict[str, str]:
        return {"value": dep}

    with create_test_client(route_handlers=[handler]) as client:
        res = client.get("/")
        assert res.status_code == 200
        assert res.json() == {"value": "hello"}
        cleanup_mock.assert_called_once()
        finally_mock.assert_called_once()
        exception_mock.assert_not_called()


@pytest.mark.parametrize("dependency_fixture", ["generator_dependency", "async_generator_dependency"])
async def test_generator_dependency_websocket(
    request: FixtureRequest,
    dependency_fixture: str,
    cleanup_mock: MagicMock,
    exception_mock: MagicMock,
    finally_mock: MagicMock,
) -> None:
    dependency = request.getfixturevalue(dependency_fixture)

    @websocket("/ws", dependencies={"dep": dependency})
    async def ws_handler(socket: WebSocket, dep: str) -> None:
        await socket.accept()
        await socket.send_json({"value": dep})
        await socket.close()

    with create_test_client(route_handlers=[ws_handler]) as client, client.websocket_connect("/ws") as ws:
        assert ws.receive_json() == {"value": "hello"}
    cleanup_mock.assert_called_once()
    finally_mock.assert_called_once()
    exception_mock.assert_not_called()


@pytest.mark.parametrize("dependency_fixture", ["generator_dependency", "async_generator_dependency"])
def test_generator_dependency_handle_exception_debug_false(
    request: FixtureRequest,
    dependency_fixture: str,
    cleanup_mock: MagicMock,
    exception_mock: MagicMock,
    finally_mock: MagicMock,
) -> None:
    dependency = request.getfixturevalue(dependency_fixture)

    @get("/", dependencies={"dep": dependency})
    def handler(dep: str) -> Dict[str, str]:
        raise ValueError("foo")

    with create_test_client(route_handlers=[handler], debug=False) as client:
        res = client.get("/")
        assert res.status_code == 500
        assert res.json() == {"detail": "Internal Server Error", "status_code": 500}
        cleanup_mock.assert_not_called()
        exception_mock.assert_called_once()
        finally_mock.assert_called_once()


@pytest.mark.parametrize("dependency_fixture", ["generator_dependency", "async_generator_dependency"])
def test_generator_dependency_exception_during_cleanup_debug_false(
    request: FixtureRequest,
    dependency_fixture: str,
    cleanup_mock: MagicMock,
    exception_mock: MagicMock,
    finally_mock: MagicMock,
) -> None:
    dependency = request.getfixturevalue(dependency_fixture)
    cleanup_mock.side_effect = Exception("foo")

    @get("/", dependencies={"dep": dependency})
    def handler(dep: str) -> Dict[str, str]:
        return {"value": dep}

    with create_test_client(route_handlers=[handler], debug=False) as client:
        res = client.get("/")
        assert res.status_code == 500
        assert res.json() == {"status_code": 500, "detail": "Internal Server Error"}
        cleanup_mock.assert_called_once()
        finally_mock.assert_called_once()


@pytest.mark.parametrize("dependency_fixture", ["generator_dependency", "async_generator_dependency"])
@pytest.mark.usefixtures("disable_warn_sync_to_thread_with_async")
def test_generator_dependency_nested(
    request: FixtureRequest,
    dependency_fixture: str,
    cleanup_mock: MagicMock,
    exception_mock: MagicMock,
    finally_mock: MagicMock,
) -> None:
    dependency = request.getfixturevalue(dependency_fixture)

    async def nested_dependency_one(generator_dep: str) -> str:
        return generator_dep

    async def nested_dependency_two(generator_dep: str, nested_one: str) -> str:
        return generator_dep + nested_one

    @get(
        "/",
        dependencies={
            "generator_dep": dependency,
            "nested_one": nested_dependency_one,
            "nested_two": nested_dependency_two,
        },
    )
    def handler(nested_two: str) -> Dict[str, str]:
        return {"value": nested_two}

    with create_test_client(route_handlers=[handler]) as client:
        res = client.get("/")
        assert res.status_code == 200
        assert res.json() == {"value": "hellohello"}
        cleanup_mock.assert_called_once()
        finally_mock.assert_called_once()
        exception_mock.assert_not_called()


@pytest.mark.parametrize("dependency_fixture", ["generator_dependency", "async_generator_dependency"])
def test_generator_dependency_nested_error_during_cleanup(
    request: FixtureRequest,
    dependency_fixture: str,
    cleanup_mock: MagicMock,
    exception_mock: MagicMock,
    finally_mock: MagicMock,
) -> None:
    dependency = request.getfixturevalue(dependency_fixture)
    cleanup_mock_no_raise = MagicMock()
    cleanup_mock.side_effect = ValueError()

    async def other_dependency(generator_dep: str) -> AsyncGenerator[str, None]:
        try:
            yield f"{generator_dep}, world"
        finally:
            cleanup_mock_no_raise()

    @get(
        "/",
        dependencies={"generator_dep": dependency, "other": other_dependency},
    )
    def handler(other: str) -> Dict[str, str]:
        return {"value": other}

    with create_test_client(route_handlers=[handler]) as client:
        res = client.get("/")
        assert res.status_code == 200
        assert res.json() == {"value": "hello, world"}
        cleanup_mock.assert_called_once()
        finally_mock.assert_called_once()
        exception_mock.assert_called_once()
        cleanup_mock_no_raise.assert_called_once()


def test_exception_on_response_thrown_into_generators() -> None:
    counter = 0

    async def dependency() -> AsyncGenerator[int, None]:
        nonlocal counter
        counter += 1
        try:
            yield counter
        finally:
            counter -= 1

    class CustomResponse(Response[str]):
        def to_asgi_response(
            self,
            *args: Any,
            **kwargs: Any,
        ) -> ASGIResponse:
            raise Exception("foo")

    @get("/", dependencies={"dep": dependency})
    def handler(dep: int) -> CustomResponse:
        return CustomResponse("")

    with create_test_client(route_handlers=[handler]) as client:
        res = client.get("/")
        assert res.status_code == 500
        assert counter == 0


def test_exception_thrown_during_cleanup_of_exception() -> None:
    counter = 0

    async def dependency() -> AsyncGenerator[int, None]:
        nonlocal counter
        counter += 1
        try:
            yield counter
        finally:
            counter -= 1
            raise ValueError()

    class CustomResponse(Response[str]):
        def to_asgi_response(
            self,
            *args: Any,
            **kwargs: Any,
        ) -> ASGIResponse:
            raise Exception("foo")

    @get("/", dependencies={"dep": dependency})
    def handler(dep: int) -> CustomResponse:
        return CustomResponse("")

    with create_test_client(route_handlers=[handler]) as client:
        res = client.get("/")
        assert res.status_code == 500
        assert counter == 0



"""
Test the app running in a subprocess
"""

import asyncio
import pathlib
import sys
from typing import AsyncIterator, Iterator

import httpx
import httpx_sse
import pytest

from litestar.testing import subprocess_async_client, subprocess_sync_client
from litestar.testing.client.subprocess_client import StartupError, run_app

if sys.platform == "win32":
    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())

ROOT = pathlib.Path(__file__).parent
APP = "demo:app"


@pytest.fixture(name="async_client")
async def fx_async_client() -> AsyncIterator[httpx.AsyncClient]:
    async with subprocess_async_client(workdir=ROOT, app=APP) as client:
        yield client


@pytest.fixture(name="sync_client")
def fx_sync_client() -> Iterator[httpx.Client]:
    with subprocess_sync_client(workdir=ROOT, app=APP) as client:
        yield client


async def test_run_app() -> None:
    """Ensure that method returns application url if started successfully"""
    with run_app(workdir=ROOT, app=APP, capture_output=False) as url:
        assert isinstance(url, str)
        assert url.startswith("http://127.0.0.1:")


async def test_run_app_exception() -> None:
    """
    Ensure that method throws a StartupError if the application fails to start.
    To simulate this, we set retry_count=0, so that we don't check if the application has started.
    """
    with pytest.raises(StartupError):
        with run_app(workdir=ROOT, app=APP, retry_count=0, capture_output=False):
            ...


async def test_subprocess_async_client(async_client: httpx.AsyncClient) -> None:
    """Demonstrates functionality of the async client with an infinite SSE source that cannot be tested with the
    regular async test client.
    """

    async with httpx_sse.aconnect_sse(async_client, "GET", "/notify/hello") as event_source:
        async for event in event_source.aiter_sse():
            assert event.data == "hello"
            break


def test_subprocess_sync_client(sync_client: httpx.Client) -> None:
    """Demonstrates functionality of the async client with an infinite SSE source that cannot be tested with the
    regular async test client.
    """

    with httpx_sse.connect_sse(sync_client, "GET", "/notify/hello") as event_source:
        for event in event_source.iter_sse():
            assert event.data == "hello"
            break


def test_log_endpoint_capture_output_true(capfd: pytest.CaptureFixture[str]) -> None:
    """With capture_output=True, subprocess print statements are captured."""
    with subprocess_sync_client(workdir=ROOT, app=APP, capture_output=True) as client:
        response = client.post("/log", params={"message": "capfd-exists-test"})
        assert response.status_code == 201

    captured = capfd.readouterr()
    assert "Received log entry: capfd-exists-test" in captured.out


def test_log_endpoint_capture_output_false(capfd: pytest.CaptureFixture[str]) -> None:
    """With capture_output=False, subprocess print statements are suppressed."""
    with subprocess_sync_client(workdir=ROOT, app=APP, capture_output=False) as client:
        response = client.post("/log", params={"message": "capfd-not-exists-test"})
        assert response.status_code == 201

    captured = capfd.readouterr()
    assert "Received log entry: capfd-not-exists-test" not in captured.out
    assert "Received log entry: capfd-not-exists-test" not in captured.err



from datetime import datetime
from typing import List
from uuid import UUID

from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column, relationship
from typing_extensions import Annotated

from litestar import Litestar, post
from litestar.dto import DTOConfig, dto_field
from litestar.plugins.sqlalchemy import SQLAlchemyDTO

from .my_lib import Base


class Address(Base):
    street: Mapped[str]
    city: Mapped[str]
    state: Mapped[str]
    zip: Mapped[str]


class Pets(Base):
    name: Mapped[str]
    user_id: Mapped[UUID] = mapped_column(ForeignKey("user.id"))


class User(Base):
    name: Mapped[str]
    password: Mapped[str] = mapped_column(info=dto_field("private"))
    created_at: Mapped[datetime] = mapped_column(info=dto_field("read-only"))
    address_id: Mapped[UUID] = mapped_column(ForeignKey("address.id"), info=dto_field("private"))
    address: Mapped[Address] = relationship(info=dto_field("read-only"))
    pets: Mapped[List[Pets]] = relationship(info=dto_field("read-only"))


UserDTO = SQLAlchemyDTO[User]
config = DTOConfig(
    include={
        "address.street",
        "pets.0.name",
    }
)
ReadUserDTO = SQLAlchemyDTO[Annotated[User, config]]


@post("/users", dto=UserDTO, return_dto=ReadUserDTO, sync_to_thread=False)
def create_user(data: User) -> User:
    data.created_at = datetime.min
    data.address = Address(street="123 Main St", city="Anytown", state="NY", zip="12345")
    data.pets = [Pets(id=1, name="Fido"), Pets(id=2, name="Spot")]
    return data


app = Litestar(route_handlers=[create_user])

# run: /users -H "Content-Type: application/json" -d '{"name":"Litestar User","password":"xyz","created_at":"2023-04-24T00:00:00Z"}'



from typing import TYPE_CHECKING, List, Type

from litestar import Controller, Litestar, Router, get
from litestar.datastructures import State
from litestar.middleware import MiddlewareProtocol

if TYPE_CHECKING:
    from litestar.types import ASGIApp, Receive, Scope, Send


def create_test_middleware(middleware_id: int) -> Type[MiddlewareProtocol]:
    class TestMiddleware(MiddlewareProtocol):
        def __init__(self, app: "ASGIApp") -> None:
            self.app = app

        async def __call__(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
            litestar_app = scope["app"]
            litestar_app.state.setdefault("middleware_calls", [])
            litestar_app.state["middleware_calls"].append(middleware_id)
            await self.app(scope, receive, send)

    return TestMiddleware


class MyController(Controller):
    path = "/controller"
    middleware = [create_test_middleware(4), create_test_middleware(5)]

    @get(
        "/handler",
        middleware=[create_test_middleware(6), create_test_middleware(7)],
    )
    async def my_handler(self, state: State) -> List[int]:
        return state["middleware_calls"]


router = Router(
    path="/router",
    route_handlers=[MyController],
    middleware=[create_test_middleware(2), create_test_middleware(3)],
)

app = Litestar(
    route_handlers=[router],
    middleware=[create_test_middleware(0), create_test_middleware(1)],
)


# run: /router/controller/handler



from typing import Any, Dict

from typing_extensions import Annotated

from litestar import Litestar, get
from litestar.di import Provide
from litestar.params import Dependency


async def provide_str() -> str:
    """Returns a string."""
    return "whoops"


@get("/", dependencies={"injected": Provide(provide_str)}, sync_to_thread=False)
def hello_world(injected: Annotated[int, Dependency(skip_validation=True)]) -> Dict[str, Any]:
    """Handler expects an `int`, but we've provided a `str`."""
    return {"hello": injected}


app = Litestar(route_handlers=[hello_world])



from click.testing import CliRunner

from litestar.cli._utils import LitestarExtensionGroup
from tests.unit.test_cli.conftest import CreateAppFileFixture

APPLICATION_WITH_CLI_PLUGIN = """
from litestar import Litestar
from litestar.plugins import CLIPluginProtocol

class CLIPlugin(CLIPluginProtocol):
    def on_cli_init(self, cli):
        @cli.command()
        def mycommand(app: Litestar):
            \"\"\"Description of plugin command\"\"\"
            print(f"App is loaded: {app is not None}")

app = Litestar(plugins=[CLIPlugin()])
"""


def test_basic_command(
    runner: CliRunner,
    create_app_file: CreateAppFileFixture,
    root_command: LitestarExtensionGroup,
) -> None:
    app_file = create_app_file("command_test_app.py", content=APPLICATION_WITH_CLI_PLUGIN)
    result = runner.invoke(root_command, ["--app", f"{app_file.stem}:app", "mycommand"])

    assert not result.exception
    assert "App is loaded: True" in result.output


def test_plugin_command_appears_in_help_message(
    runner: CliRunner,
    create_app_file: CreateAppFileFixture,
    root_command: LitestarExtensionGroup,
) -> None:
    app_file = create_app_file("command_test_app.py", content=APPLICATION_WITH_CLI_PLUGIN)
    result = runner.invoke(root_command, ["--app", f"{app_file.stem}:app", "--help"])

    assert not result.exception
    assert "mycommand" in result.output
    assert "Description of plugin command" in result.output


def test_format_help_loads_plugins_before_rendering(
    runner: CliRunner,
    create_app_file: CreateAppFileFixture,
    root_command: LitestarExtensionGroup,
) -> None:
    """Test that format_help method correctly loads plugins before rendering help.

    This specifically tests the fix for rich-click where format_help was called
    before _prepare(), causing plugin commands to not appear in help output.
    """
    app_file = create_app_file("format_help_test_app.py", content=APPLICATION_WITH_CLI_PLUGIN)

    # Test by invoking help which internally calls format_help
    result = runner.invoke(root_command, ["--app", f"{app_file.stem}:app", "--help"])

    # Ensure the command succeeded
    assert result.exit_code == 0, f"Command failed with output: {result.output}"

    # Verify that plugin commands are included in the help output
    assert "mycommand" in result.output, "Plugin command should appear in help output"
    assert "Description of plugin command" in result.output, "Plugin command description should appear in help output"

    # Additional verification: ensure the plugin command is in the Commands section
    lines = result.output.split("\n")
    commands_section_found = False
    for line in lines:
        if "Commands" in line:
            commands_section_found = True
        if commands_section_found and "mycommand" in line:
            break
    else:
        assert False, "Plugin command not found in Commands section of help output"



from dataclasses import dataclass, field
from uuid import UUID, uuid4

from litestar import Litestar, post
from litestar.dto import DataclassDTO


@dataclass
class User:
    name: str
    email: str
    age: int
    id: UUID = field(default_factory=uuid4)


UserDTO = DataclassDTO[User]


@post(dto=UserDTO, return_dto=None, sync_to_thread=False)
def create_user(data: User) -> bytes:
    return data.name.encode(encoding="utf-8")


app = Litestar([create_user])



from __future__ import annotations

import itertools
from dataclasses import replace
from typing import TYPE_CHECKING
from unittest.mock import ANY

import pytest
from msgspec import Meta, Struct, field
from typing_extensions import Annotated

from litestar import Litestar, post
from litestar.dto import DTOField, Mark, MsgspecDTO, dto_field
from litestar.dto.data_structures import DTOFieldDefinition
from litestar.typing import FieldDefinition

if TYPE_CHECKING:
    from typing import Callable


@pytest.fixture
def expected_field_defs(int_factory: Callable[[], int]) -> list[DTOFieldDefinition]:
    return [
        DTOFieldDefinition.from_field_definition(
            field_definition=FieldDefinition.from_kwarg(
                annotation=int,
                name="a",
            ),
            model_name=ANY,
            default_factory=None,
            dto_field=DTOField(),
        ),
        replace(
            DTOFieldDefinition.from_field_definition(
                field_definition=FieldDefinition.from_kwarg(
                    annotation=int,
                    name="b",
                ),
                model_name=ANY,
                default_factory=None,
                dto_field=DTOField(mark=Mark.READ_ONLY),
            ),
            metadata=ANY,
            type_wrappers=ANY,
            raw=ANY,
            kwarg_definition=ANY,
        ),
        replace(
            DTOFieldDefinition.from_field_definition(
                field_definition=FieldDefinition.from_kwarg(
                    annotation=int,
                    name="c",
                ),
                model_name=ANY,
                default_factory=None,
                dto_field=DTOField(),
            ),
            metadata=ANY,
            type_wrappers=ANY,
            raw=ANY,
            kwarg_definition=ANY,
        ),
        replace(
            DTOFieldDefinition.from_field_definition(
                field_definition=FieldDefinition.from_kwarg(
                    annotation=int,
                    name="d",
                    default=1,
                ),
                model_name=ANY,
                default_factory=None,
                dto_field=DTOField(),
            ),
            metadata=ANY,
            type_wrappers=ANY,
            raw=ANY,
            kwarg_definition=ANY,
        ),
        replace(
            DTOFieldDefinition.from_field_definition(
                field_definition=FieldDefinition.from_kwarg(
                    annotation=int,
                    name="e",
                ),
                model_name=ANY,
                default_factory=int_factory,
                dto_field=DTOField(),
            ),
            metadata=ANY,
            type_wrappers=ANY,
            raw=ANY,
            kwarg_definition=ANY,
        ),
        replace(
            DTOFieldDefinition.from_field_definition(
                field_definition=FieldDefinition.from_kwarg(
                    annotation=str,
                    name="computed",
                ),
                model_name=ANY,
                default_factory=None,
                dto_field=DTOField(mark="read-only"),
            ),
            metadata=ANY,
            type_wrappers=ANY,
            raw=ANY,
            kwarg_definition=ANY,
        ),
    ]


def test_field_definition_generation(
    int_factory: Callable[[], int], expected_field_defs: list[DTOFieldDefinition]
) -> None:
    class TestStruct(Struct):
        a: int
        b: Annotated[int, Meta(extra=dto_field("read-only"))]
        c: Annotated[int, Meta(gt=1)]
        d: int = field(default=1)
        e: int = field(default_factory=int_factory)

        @property
        def computed(self) -> str:
            return "i am computed"

        @property
        def _private_computed(self) -> str:
            return ""

    field_defs = list(MsgspecDTO.generate_field_definitions(TestStruct))
    assert field_defs[0].model_name == "TestStruct"
    for field_def, exp in itertools.zip_longest(expected_field_defs, field_defs, fillvalue=None):
        assert field_def == exp


def test_detect_nested_field() -> None:
    class TestStruct(Struct):
        a: int

    class NotStruct:
        pass

    assert MsgspecDTO.detect_nested_field(FieldDefinition.from_annotation(TestStruct)) is True
    assert MsgspecDTO.detect_nested_field(FieldDefinition.from_annotation(NotStruct)) is False


ReadOnlyInt = Annotated[int, DTOField("read-only")]


def test_msgspec_dto_annotated_dto_field() -> None:
    class Model(Struct):
        a: Annotated[int, DTOField("read-only")]
        b: ReadOnlyInt

    dto_type = MsgspecDTO[Model]
    fields = list(dto_type.generate_field_definitions(Model))
    assert fields[0].dto_field == DTOField("read-only")
    assert fields[1].dto_field == DTOField("read-only")


def test_tag_field_included_in_schema() -> None:
    # default tag field, default tag value
    class Model(Struct, tag=True):
        regular_field: str

    # default tag field, custom tag value
    class Model2(Struct, tag=2):
        regular_field: str

    # custom tag field, custom tag value
    class Model3(Struct, tag_field="foo", tag="bar"):
        regular_field: str

    @post("/1")
    def handler(data: Model) -> None:
        return None

    @post("/2")
    def handler_2(data: Model2) -> None:
        return None

    @post("/3")
    def handler_3(data: Model3) -> None:
        return None

    components = Litestar(
        [handler, handler_2, handler_3],
        signature_types=[Model, Model2, Model3],
    ).openapi_schema.components.to_schema()["schemas"]

    assert components["test_tag_field_included_in_schema.Model"] == {
        "properties": {
            "regular_field": {"type": "string"},
            "type": {"type": "string", "const": "Model"},
        },
        "type": "object",
        "required": ["regular_field", "type"],
        "title": "Model",
    }

    assert components["test_tag_field_included_in_schema.Model2"] == {
        "properties": {
            "regular_field": {"type": "string"},
            "type": {"type": "integer", "const": 2},
        },
        "type": "object",
        "required": ["regular_field", "type"],
        "title": "Model2",
    }

    assert components["test_tag_field_included_in_schema.Model3"] == {
        "properties": {
            "regular_field": {"type": "string"},
            "foo": {"type": "string", "const": "bar"},
        },
        "type": "object",
        "required": ["foo", "regular_field"],
        "title": "Model3",
    }



from litestar.openapi.plugins import ScalarRenderPlugin

scalar_plugin = ScalarRenderPlugin(version="1.19.5", path="/scalar")

# Example demonstrating custom Scalar render options
# For more configuration options, see: https://guides.scalar.com/scalar/scalar-api-references/configuration
scalar_plugin_with_options = ScalarRenderPlugin(
    options={
        # Hide the sidebar to give more space to the main content
        "showSidebar": False
    }
)
